# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.publish.formatters.field_mappers import FieldMapper
import superdesk
import logging

logger = logging.getLogger(__name__)


class SelectorcodeMapper(FieldMapper):
    SELECTOR_CODES = {'sportdmedia': {'ipnews': '0ah 0fh 0hw 0nl 0px psd pxd pxx'},
                      'rnewsdnt': {'ipnews': '0fh bn8 bx8 rn8 rx8'},
                      'newsnz': {'newscentre': 'nxx', 'ipnews': 'nxx', 'notes': 'nxx'},
                      'newsdnotqld': {'ipnews': '0ah 0fh 0hw 0nl 0px an2 an3 an5 an6 an7 an8 ax2 ax3 ax5 ax6 ax7 ax8 '
                                      'pn2 pn3 pn5 pn6 pn7 px0 px2 px3 px5 px6 px7 px8'},
                      'rnewsdnotnsw': {
                          'ipnews': 'bn3 bn4 bn5 bn6 bn7 bx3 bx4 bx5 bx6 bx7 bx8 rn3 rn4 rn5 rn6 rn7 rx3 rx4 rx5 rx6 '
                          'rx7 rx8'},
                      'rnewsdnotwa': {
                          'ipnews': '0fh bn2 bn3 bn4 bn5 bn7 bx2 bx3 bx4 bx5 bx7 bx8 rn2 rn3 rn4 rn5 rn7 rx2 rx3 rx4 '
                          'rx5 rx7 rx8'},
                      'sportdsa': {'ipnews': '0ah 0fh 0nl 3ff 3nz 3pn 3rk as5 ax5 ps5 px5'},
                      'newsd': {'newscentre': 'and axd axx pnd pxd pxx',
                                'ipnews': '0ah 0fh 0hw 0ir 0nl 0px and axd axx az pnd pxd pxx',
                                'notes': 'and axd axx pnd pxd pxx'},
                      'newsdvic': {'ipnews': '0fh 0hw 0ir an3 ax3 az pn3 px3'},
                      'rnewsdvic': {'ipnews': '0fh bn3 bx3 rn3 rx3'},
                      'newsdnottas': {
                          'ipnews': '0ah 0fh 0hw 0nl 0px an2 an3 an4 an5 an6 an8 ax2 ax3 ax4 ax5 ax6 ax8 pn2 pn3 pn4 '
                          'pn5 pn6 px0 px2 px3 px4 px5 px6 px8'},
                      'newsdnsw': {'ipnews': '0fh 0ir 0nl an2 ax2 az pn2 px2'},
                      'rnewsdnotsa': {
                          'ipnews': '0fh bn2 bn3 bn4 bn6 bn7 bx2 bx3 bx4 bx6 bx7 bx8 rn2 rn3 rn4 rn6 rn7 rx2 rx3 rx4 '
                          'rx6 rx7 rx8'},
                      'sfinancei': {'newscentre': 'afi afx axx pfi pfx pxx sfi sfx sxx',
                                    'ipnews': '0ah 0fh 0fr 0nl sfi sfx sxx',
                                    'notes': 'afi afx axx pfi pfx pxx sfi sfx sxx'},
                      'newsdwa': {'ipnews': '0ir 0px an6 ax6 az pn6 px6'},
                      'newsdnotnt': {
                          'ipnews': '0ah 0fh 0hw 0nl an2 an3 an4 an5 an6 an7 ax2 ax3 ax4 ax5 ax6 ax7 pn2 pn3 pn4 pn5 '
                          'pn6 pn7 px0 px2 px3 px4 px5 px6 px7'},
                      'newsi': {'newscentre': 'ani axi axx pni pxi pxx',
                                'ipnews': '0ah 0fh 0nl ani axi axx pni pxi pxx',
                                'notes': 'ani axi axx pni pxi pxx'},
                      'rnewsdtas': {'ipnews': '0fh bn7 bx7 rn7 rx7'},
                      'sporti': {'newscentre': 'asi axi axx psi pxi pxx',
                                 'ipnews': '0ah 0fh 0hw 0nl asi axi axx psi pxi pxx',
                                 'notes': 'asi axi axx psi pxi pxx'},
                      'sportdvic': {'ipnews': '0ab 0fh 0hw 0nl 3ff 3nz 3pn 3rk as3 ax3 ps3 px3'},
                      'sportdnsw': {'ipnews': '0fh 0nl 3ff 3nz 3pn 3rk as2 ax2 ps2 px2'},
                      'sportd': {'newscentre': 'asd axd axx psd pxd pxx',
                                 'ipnews': '0ah 0fh 0hw 0nl 0px 3ff 3nz 3pn 3rk asd axd axx psd pxd pxx',
                                 'notes': 'asd axd axx psd pxd pxx'},
                      'newsdsa': {'ipnews': '0ah 0ir an5 ax5 az pn5 px5'},
                      'monitor': {'newscentre': 'and bnd bxd bxx rnd rxd rxx',
                                  'ipnews': '0ah 0fh 0hw 0nl 0px bnd bxd bxx pnd pxd pxx rnd rxd rxx',
                                  'notes': 'bnd bxd bxx rnd rxd rxx'},
                      'csporti': {'newscentre': 'csi cxi cxx', 'ipnews': 'csi cxi cxx', 'notes': 'csi cxi cxx'},
                      'financei': {'newscentre': '2in afi axi axx pfi pxi pxx',
                                   'ipnews': '0ah 0fh 0nl 2in afi axi axx pfi pxi pxx',
                                   'notes': '2in afi axi axx pfi pxi pxx'},
                      'newsdqld': {'ipnews': '0ir an4 ax4 az pn4 px4'},
                      'cnewsi': {'newscentre': 'cni cxi cxx', 'ipnews': 'cni cxi cxx', 'notes': 'cni cxi cxx'},
                      'newsdnotnsw': {
                          'ipnews': '0ah 0fh 0hw 0px an3 an4 an5 an6 an7 an8 ax3 ax4 ax5 ax6 ax7 ax8 pn3 pn4 pn5 pn6 '
                          'pn7 px0 px3 px4 px5 px6 px7 px8'},
                      'rnewsd': {'newscentre': 'and bnd bxd bxx rnd rxd rxx', 'ipnews': '0fh bnd bxd bxx rnd rxd rxx',
                                 'notes': 'bnd bxd bxx rnd rxd rxx'},
                      'rnewsdnotnt': {
                          'ipnews': '0fh bn2 bn3 bn4 bn5 bn6 bn7 bx2 bx3 bx4 bx5 bx6 bx7 rn2 rn3 rn4 rn5 rn6 rn7 '
                          'rx2 rx3 rx4 rx5 rx6 rx7'},
                      'eprem': {'newscentre': 'jxx',
                                'ipnews': '0ah 0al 0fh 0hw 0nh 0nl 0pb 0px 0tb 0tc 0tn 0wm 3ff 3pn jxx'},
                      'newsdtas': {'ipnews': '0ir an7 ax7 az pn7 px7'},
                      'sfinanced': {'newscentre': '2i1 2in 2mr afd sfd sfx sxx',
                                    'ipnews': '0ah 0fh 0fr 0hw 0nl 2in 2mr i1 in mr sfd sfx sxx',
                                    'notes': '2i1 2in 2mr sfd sfx sxx'},
                      'csportd': {'newscentre': 'csd cxd cxx',
                                  'ipnews': 'csd cxd cxx',
                                  'notes': 'csd cxd cxx'},
                      'fronters': {'newscentre': 'and axd axx bnd bxd bxx pnd pxd pxx rnd rxd rxx',
                                   'ipnews': '0ah 0fh 0hw 0ir 0nl 0px 0qn 3ff 3nz 3pn 3rk and axd axx az bnd bxd '
                                   'bxx pnd pxd pxx rnd rxd rxx',
                                   'notes': 'and axd axx bnd bxd bxx pnd pxd pxx rnd rxd rxx'},
                      'rnewsdsa': {'ipnews': '0fh bn5 bx5 rn5 rx5'},
                      'sportdwa': {'ipnews': '0fh 0nl 0px 3ff 3nz 3pn 3rk as6 ax6 ps6 px6'},
                      'rnewsdnsw': {'ipnews': '0fh bn2 bx2 rn2 rx2'},
                      'rnewsdnottas': {
                          'ipnews': '0fh bn2 bn3 bn4 bn5 bn6 bx2 bx3 bx4 bx5 bx6 bx8 rn2 rn3 rn4 rn5 rn6 rx2 rx3 '
                          'rx4 rx5 rx6 rx8'},
                      'printsubs': {'newscentre': 'aax pad pai pax',
                                    'ipnews': '0ah 0fh 0hw 0nl 0px 0qn aax pad pai pax',
                                    'notes': 'aax pad pai pax'},
                      'sportdtas': {'ipnews': '0fh 0nl 3ff 3nz 3pn 3rk as7 ax7 ps7 px7'},
                      'cnewsd': {'newscentre': 'cnd cxd cxx',
                                 'ipnews': 'cnd cxd cxx',
                                 'notes': 'cnd cxd cxx'},
                      'newsdnotsa': {
                          'ipnews': '0fh 0hw 0nl 0px an2 an3 an4 an6 an7 an8 ax2 ax3 ax4 ax6 ax7 ax8 pn2 pn3 pn4 pn6 '
                          'pn7 px0 px2 px3 px4 px6 px7 px8'},
                      'rnewsdqld': {'ipnews': '0fh bn4 bx4 rn4 rx4'},
                      'newsdnotvic': {
                          'ipnews': '0ah 0fh 0nl 0px an2 an4 an5 an6 an7 an8 ax2 ax4 ax5 ax6 ax7 ax8 pn2 pn4 pn5 '
                          'pn6 pn7 px0 px2 px4 px5 px6 px7 px8'},
                      'newsdnotwa': {
                          'ipnews': '0ah 0fh 0hw 0nl an2 an3 an4 an5 an7 an8 ax2 ax3 ax4 ax5 ax7 ax8 pn2 pn3 pn4 pn5 '
                          'pn7 px0 px2 px3 px4 px5 px7 px8'},
                      'rnewsdnotqld': {
                          'ipnews': '0fh bn2 bn3 bn5 bn6 bn7 bx2 bx3 bx5 bx6 bx7 bx8 rn2 rn3 rn5 rn6 rn7 rx2 rx3 rx5 '
                          'rx6 rx7 rx8'},
                      'rnewsdwa': {'ipnews': '0fh bn6 bx6 rn6 rx6'},
                      'rnewsdnotvic': {
                          'ipnews': '0fh bn2 bn4 bn5 bn6 bn7 bx2 bx4 bx5 bx6 bx7 bx8 rn2 rn4 rn5 rn6 rn7 rx2 rx4 '
                          'rx5 rx6 rx7 rx8'},
                      'sportdqld': {'ipnews': '0fh 0nl 0qn 3ff 3nz 3pn 3rk as4 ax4 ps4 px4'},
                      'cfinancei': {'newscentre': 'cfi cxi cxx',
                                    'ipnews': 'cfi cxi cxx',
                                    'notes': 'cfi cxi cxx'}}

    def map(self, article, category, **kwargs):
        handlers = {
            'NATIONAL': self._set_ndx_selector_codes,
            'SPORT': self._set_sdx_selector_codes,
            'FINANCE': self._set_fdx_selector_codes,
        }

        desk_id = article.get('task', {}).get('desk', None)
        if desk_id:
            desk = superdesk.get_resource_service('desks').find_one(req=None, _id=desk_id)
            subscriber = kwargs['subscriber']
            formatted_item = kwargs['formatted_item']
            handler = handlers.get(desk['name'].upper(), lambda *args: None)
            handler(article, subscriber['name'], formatted_item, category)

    def _get_geo_abbreviation(self, name):
        """
        passed the name that is kept in the article uses it to find the value from the vocabulary
        :param name:
        :return: the abbreviatoin for the geographical restriction
        """
        geo_resource = superdesk.get_resource_service('vocabularies').find_one(req=None,
                                                                               _id='geographical_restrictions')
        item = next((i for i in geo_resource.get('items') if i['name'] == name), None)
        if item:
            return item.get('value')
        else:
            return ''

    def set_selectors_for_dg(self, article, subscriber_name, *dist_groups):
        """
        Test if there is a geo block, if there is calculate the required selector codes, if not just join as normal
        :param article:
        :param subscriber_name:
        :param dist_groups:
        :return: Space seperated string of selector codes
        """
        # if the is a geographical block
        if 'targeted_for' in article and len(article.get('targeted_for')) > 0:
            selector_set = set()
            # scan each block clause
            for geo_block in article.get('targeted_for'):
                # scan each distribution group
                for dg in dist_groups:
                    if geo_block.get('allow'):
                        geo_value = dg + self._get_geo_abbreviation(geo_block.get('name')).lower()
                        if geo_value in self.SELECTOR_CODES and subscriber_name.lower() in \
                                self.SELECTOR_CODES[geo_value]:
                            selector_set = selector_set | \
                                set(self.SELECTOR_CODES[geo_value][subscriber_name.lower()].split(' '))
                    else:
                        geo_value = dg + 'not' + self._get_geo_abbreviation(geo_block.get('name')).lower()
                        if geo_value in self.SELECTOR_CODES and subscriber_name.lower() in \
                                self.SELECTOR_CODES[geo_value]:
                            if selector_set:
                                selector_set = selector_set & \
                                    set(self.SELECTOR_CODES[geo_value][subscriber_name.lower()].split(' '))
                            else:
                                selector_set = set(self.SELECTOR_CODES[geo_value][subscriber_name.lower()].split(' '))
            return ' '.join((list(selector_set)))
        else:  # normal case just join the distribution groups
            return self._join_selector_codes(subscriber_name, *dist_groups)

    def _set_ndx_selector_codes(self, article, subscriber_name, odbc_item, category):
        slugline = article['slugline'].upper()

        if 'MONITOR' in slugline:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'monitor')
            return
        if 'DIARY' in slugline:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name,
                                                                    'printsubs', 'cnewsd', 'rnewsd')
            return
        if 'FRONTERS' in slugline:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'fronters')
            return
        if category == 'A' and article['urgency'] > 3:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'newsd')
            return
        if category == 'A' and article['urgency'] <= 3:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'newsd', 'cnewsd')
            return
        if category == 'I' and article['source'] == 'AAP':
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'newsd', 'cnewsd')
            return
        if category == 'I' and article['source'] != 'AAP' and article['urgency'] <= 3:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'newsi', 'cnewsi')
            return
        if category == 'I' and article['source'] != 'AAP' and article['urgency'] > 3:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'newsi')
            return
        if category == 'F' and article['source'] != 'AAP':
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name,
                                                                    'financei', 'cfinancei', 'sfinancei', 'cnewsi')
            return
        if category == 'F' and article['source'] == 'AAP':
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'newsd', 'cnewsd')
            return
        if category == 'E' and article['source'] == 'AAP' \
                and article['urgency'] < 3 and self._is_in_subject(article, '010'):
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name,
                                                                    'eprem', 'newsd', 'cnewsd')
            return
        if category == 'E' and article['source'] != 'AAP' \
                and article['urgency'] < 3 and self._is_in_subject(article, '010'):
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name,
                                                                    'eprem', 'newsi', 'cnewsi')
            return
        if category == 'E' and article['urgency'] >= 3 and self._is_in_subject(article, '010'):
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'eprem')
            return
        if category == 'N' and article['source'] != 'NZN':
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'newsz')
            return
        odbc_item['selector_codes'] = '   '
        logger.error('No Selector code derived from National desk')

    def _set_sdx_selector_codes(self, article, subscriber_name, odbc_item, category):
        if 'NEWSLIST' in article['slugline'].upper():
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'sportmedia', 'sportseds')
            return
        if category == 'T' and article['source'] == 'AAP' and article['urgency'] > 3:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'sportd')
            return
        if category == 'T' and article['source'] == 'AAP' and article['urgency'] <= 3:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'sportd', 'csportd')
            return
        if category == 'S' and article['source'] == 'AAP':
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name,
                                                                    'sporti', 'csporti', 'csportd')
            return
        if category == 'S' and article['source'] != 'AAP' and article['urgency'] <= 3:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'sporti', 'csporti')
            return
        if category == 'S' and article['source'] != 'AAP' and article['urgency'] > 3:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'sporti')
            return
        odbc_item['selector_codes'] = '   '
        logger.error('No Selector code derived from sport desk')

    def _set_fdx_selector_codes(self, article, subscriber_name, odbc_item, category):
        if category == 'F' and article['priority'] in ['N'] and article['urgency'] > 3:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name, 'sfinanced')
            return
        if category == 'F' and article['source'] == 'AAP' and article['priority'] in ['N', 'M']:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name,
                                                                    'financed', 'cfinanced', 'sfinanced')
            return
        if category == 'F' and article['source'] != 'AAP':
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name,
                                                                    'financei', 'cfinancei', 'sfinancei')
            return
        if category == 'F' and article['source'] == 'AAP' \
                and article.get('locator', '') not in ['N', 'M']:
            odbc_item['selector_codes'] = self.set_selectors_for_dg(article, subscriber_name,
                                                                    'financei', 'cfinancei', 'sfinancei')
            return
        odbc_item['selector_codes'] = '   '
        logger.error('No Selector code derived from Finance desk')

    def _join_selector_codes(self, subscriber_name, *args):
        codes = []
        for arg in args:
            if arg in self.SELECTOR_CODES and subscriber_name.lower() in self.SELECTOR_CODES[arg]:
                codes.extend(self.SELECTOR_CODES[arg][subscriber_name.lower()].split())
        return ' '.join(list(set(codes)))

    def _is_in_subject(self, article, qcode):
        def compare(code1, code2):
            return code1[:len(code2)] == code2

        return any(compare(s['qcode'], qcode) for s in article.get('subject', []))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from eve.utils import date_to_str, config
import superdesk
from superdesk.celery_task_utils import is_task_running, mark_task_as_not_running
from superdesk.utc import utcnow
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE
from apps.archive.commands import get_overdue_scheduled_items

logger = logging.getLogger(__name__)

REMOVE_SPIKE_DEFAULT = {'minutes': 30}
UPDATE_OVERDUE_SCHEDULED_DEFAULT = {'minutes': 10}


class UpdateOverdueScheduledPublishedContent(superdesk.Command):
    """
    Update the overdue scheduled stories
    """

    def run(self):
        self.update_overdue_scheduled()

    def update_overdue_scheduled(self):
        """
        Updates the overdue scheduled content on published collection.
        """

        logger.info('Updating overdue scheduled content')

        if is_task_running("publish", "update_overdue_scheduled", UPDATE_OVERDUE_SCHEDULED_DEFAULT):
            return

        try:
            now = date_to_str(utcnow())
            items = get_overdue_scheduled_items(now, 'published')

            for item in items:
                logger.info('updating overdue scheduled article with id {} and headline {} -- expired on: {} now: {}'.
                            format(item[config.ID_FIELD], item['headline'], item['publish_schedule'], now))

                superdesk.get_resource_service('published').\
                    update_published_items(item['item_id'], ITEM_STATE, CONTENT_STATE.PUBLISHED)
        finally:
            mark_task_as_not_running("publish", "update_overdue_scheduled")


superdesk.command('publish:remove_overdue_scheduled', UpdateOverdueScheduledPublishedContent())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from flask import request

from superdesk import get_resource_service, Service, config
from superdesk.metadata.item import ITEM_STATE, EMBARGO, CONTENT_STATE
from superdesk.resource import Resource, build_custom_hateoas
from apps.archive.common import CUSTOM_HATEOAS, ITEM_CREATE, ARCHIVE
from superdesk.metadata.utils import item_url
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from apps.packages.takes_package_service import TakesPackageService
from apps.tasks import send_to

logger = logging.getLogger(__name__)


class ArchiveRewriteResource(Resource):
    endpoint_name = 'archive_rewrite'
    resource_title = endpoint_name

    schema = {
        'desk_id': {"type": "string"}
    }

    url = 'archive/<{0}:original_id>/rewrite'.format(item_url)
    resource_methods = ['POST']
    privileges = {'POST': 'rewrite'}


class ArchiveRewriteService(Service):
    def create(self, docs, **kwargs):
        original_id = request.view_args['original_id']
        archive_service = get_resource_service(ARCHIVE)
        original = archive_service.find_one(req=None, _id=original_id)
        self._validate_rewrite(original)
        digital = TakesPackageService().get_take_package(original)
        rewrite = self._create_rewrite_article(original, digital)
        ids = archive_service.post([rewrite])
        build_custom_hateoas(CUSTOM_HATEOAS, rewrite)
        self._add_rewritten_flag(original, digital, rewrite)
        get_resource_service('archive_broadcast').on_broadcast_master_updated(ITEM_CREATE,
                                                                              item=original,
                                                                              rewrite_id=ids[0])
        return [rewrite]

    def _validate_rewrite(self, original):
        """
        Validates the article to be rewritten
        :param original: article to be rewritten
        :raises: SuperdeskApiError
        """
        if not original:
            raise SuperdeskApiError.notFoundError(message='Cannot find the article')

        if original.get(EMBARGO):
            raise SuperdeskApiError.badRequestError("Rewrite of an Item having embargo isn't possible")

        if not original.get('event_id'):
            raise SuperdeskApiError.notFoundError(message='Event id does not exist')

        if get_resource_service('published').is_rewritten_before(original[config.ID_FIELD]):
            raise SuperdeskApiError.badRequestError(message='Article has been rewritten before !')

        if not is_workflow_state_transition_valid('rewrite', original[ITEM_STATE]):
            raise InvalidStateTransitionError()

        if not TakesPackageService().is_last_takes_package_item(original):
            raise SuperdeskApiError.badRequestError(message="Only last take of the package can be rewritten.")

    def _create_rewrite_article(self, original, digital):
        """
        Creates a new story and sets the metadata from original and digital
        :param original: original story
        :param digital: digital version of the story
        :return:new story
        """
        rewrite = dict()
        fields = ['family_id', 'abstract', 'anpa_category', 'pubstatus', 'slugline', 'urgency', 'subject', 'priority',
                  'byline', 'dateline', 'headline', 'event_id', 'place', 'flags', 'genre', 'body_footer',
                  'company_codes']

        for field in fields:
            if original.get(field):
                rewrite[field] = original[field]

        if digital:  # check if there's digital
            rewrite['rewrite_of'] = digital[config.ID_FIELD]
        else:  # if not use original's id
            rewrite['rewrite_of'] = original[config.ID_FIELD]

        send_to(doc=rewrite, desk_id=original['task']['desk'], default_stage='working_stage')
        rewrite[ITEM_STATE] = CONTENT_STATE.PROGRESS
        self._set_take_key(rewrite, original.get('event_id'))
        return rewrite

    def _add_rewritten_flag(self, original, digital, rewrite):
        """ Adds rewritten_by field to the existing published items """
        get_resource_service('published').update_published_items(original[config.ID_FIELD],
                                                                 'rewritten_by', rewrite[config.ID_FIELD])
        if digital:
            get_resource_service('published').update_published_items(digital[config.ID_FIELD], 'rewritten_by',
                                                                     rewrite[config.ID_FIELD])

        # modify the original item as well.
        get_resource_service(ARCHIVE).system_update(original[config.ID_FIELD],
                                                    {'rewritten_by': rewrite[config.ID_FIELD]}, original)

    def _clear_rewritten_flag(self, event_id, rewrite_id):
        """ Clears rewritten_by field from the existing published items """
        publish_service = get_resource_service('published')
        archive_service = get_resource_service(ARCHIVE)
        published_rewritten_stories = publish_service.get_rewritten_items_by_event_story(event_id, rewrite_id)
        processed_items = set()
        for doc in published_rewritten_stories:
            publish_service.update_published_items(doc['item_id'], 'rewritten_by', None)
            if doc['item_id'] not in processed_items:
                # clear the flag from the archive as well.
                archive_item = archive_service.find_one(req=None, _id=doc['item_id'])
                archive_service.system_update(doc['item_id'], {'rewritten_by': None}, archive_item)
                processed_items.add(doc['item_id'])

    def _set_take_key(self, rewrite, event_id):
        """
        Sets the anpa take key of the rewrite with ordinal
        :param rewrite: rewrite story
        :param event_id: event id
        """
        published_digital_stories = get_resource_service('published'). \
            get_rewritten_take_packages_per_event(event_id)

        digital_count = published_digital_stories.count()
        if digital_count > 0:
            ordinal = self._get_ordinal(digital_count + 1)
            rewrite['anpa_take_key'] = '{} update'.format(ordinal)
        else:
            rewrite['anpa_take_key'] = 'update'

    def _get_ordinal(self, n):
        """ Returns the ordinal value of the given int """
        if 10 <= n % 100 < 20:
            return str(n) + 'th'
        else:
            return str(n) + {1: 'st', 2: 'nd', 3: 'rd'}.get(n % 10, "th")




from datetime import datetime, timedelta
from steps import when, then, get_json_data, parse_date


@when('we run create content task')
def when_we_run_create_content_task(context):
    from apps.templates import create_scheduled_content
    now = datetime.now() + timedelta(days=8)
    with context.app.app_context():
        create_scheduled_content(now)


@then('next run is on monday "{time}"')
def then_next_run_is_on_monday(context, time):
    data = get_json_data(context.response)
    next_run = parse_date(data.get('next_run'))
    assert isinstance(next_run, datetime)
    assert next_run.weekday() == 0
    assert next_run.strftime('%H%M') == time
    assert next_run.strftime('%S') == '00', 'there should be no seconds, but it was %s' % (next_run.strftime('%S'), )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest

from unittest import mock
from unittest.mock import MagicMock

from superdesk.publish.publish_content import STATE_PENDING


class TransmitItemsTestCase(unittest.TestCase):
    """Tests for the transmit_items() function."""

    def setUp(self):
        try:
            from superdesk.publish.publish_content import transmit_items
        except ImportError:
            self.fail("Could not import function under test (transmit_items).")
        else:
            self.func_under_test = transmit_items

    @mock.patch('superdesk.publish.publish_content.get_resource_service')
    @mock.patch(
        'superdesk.publish.publish_content.is_on_time',
        return_value=True)
    def test_marks_items_failed_to_transmit_as_pending(self, *mocks):

        fake_get_service = mocks[1]
        fake_get_service().patch.side_effect = Exception('Error patching item')

        orig_item = MagicMock(name='orig_item')  # item's original state in DB
        fake_get_service().find_one.return_value = orig_item

        item_1 = {
            '_id': 'item_1',
            'destination': {}
        }

        queue_items = [item_1]
        self.func_under_test(queue_items)

        fake_get_service().system_update.assert_called_with(
            'item_1', {'state': STATE_PENDING}, orig_item)

    @mock.patch('superdesk.publish.publish_content.logger')
    @mock.patch('superdesk.publish.publish_content.get_resource_service')
    @mock.patch(
        'superdesk.publish.publish_content.is_on_time',
        return_value=True)
    def test_logs_error_even_when_marking_failed_items_fails(self, *mocks):

        fake_get_service = mocks[1]
        fake_get_service().patch.side_effect = Exception('Error patching item')
        fake_get_service().find_one.return_value = MagicMock(name='orig_item')
        fake_get_service().system_update.side_effect = Exception('Update error')

        item_1 = {
            '_id': 'item_1',
            'destination': {}
        }
        queue_items = [item_1]

        self.func_under_test(queue_items)

        fake_logger = mocks[2]
        expected_msg = (
            'Failed to set the publish queue item back to '
            '"pending" state: item_1')
        fake_logger.error.assert_any_call(expected_msg)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from superdesk import get_resource_service
from superdesk.notification import push_notification
from superdesk.resource import Resource
from superdesk.services import BaseService

logger = logging.getLogger(__name__)


class PublishQueueResource(Resource):
    schema = {
        'item_id': Resource.rel('archive', type='string'),
        'item_version': {'type': 'integer', 'nullable': False},

        'formatted_item': {'type': 'string', 'nullable': False},
        'subscriber_id': Resource.rel('subscribers'),
        'destination': {
            'type': 'dict',
            'schema': {
                'name': {'type': 'string', 'required': True, 'empty': False},
                'format': {'type': 'string', 'required': True},
                'delivery_type': {'type': 'string', 'required': True},
                'config': {'type': 'dict'}
            }
        },
        'published_seq_num': {
            'type': 'integer'
        },
        'publish_schedule': {
            'type': 'datetime'
        },
        'publishing_action': {
            'type': 'string'
        },
        'unique_name': {
            'type': 'string'
        },
        'content_type': {
            'type': 'string'
        },
        'headline': {
            'type': 'string'
        },

        'transmit_started_at': {
            'type': 'datetime'
        },
        'completed_at': {
            'type': 'datetime'
        },
        'state': {
            'type': 'string',
            'allowed': ['pending', 'in-progress', 'success', 'canceled', 'error'],
            'nullable': False
        },
        'error_message': {
            'type': 'string'
        }
    }

    additional_lookup = {
        'url': 'regex("[\w,.:-]+")',
        'field': 'item_id'
    }

    datasource = {'default_sort': [('_created', -1)]}
    privileges = {'POST': 'publish_queue', 'PATCH': 'publish_queue'}


class PublishQueueService(BaseService):

    def on_create(self, docs):
        subscriber_service = get_resource_service('subscribers')

        for doc in docs:
            doc['state'] = 'pending'

            if 'published_seq_num' not in doc:
                subscriber = subscriber_service.find_one(req=None, _id=doc['subscriber_id'])
                doc['published_seq_num'] = subscriber_service.generate_sequence_number(subscriber)

    def on_updated(self, updates, original):
        if updates.get('state', '') != original.get('state', ''):
            push_notification('publish_queue:update',
                              queue_id=str(original['_id']),
                              completed_at=(updates.get('completed_at').isoformat()
                                            if updates.get('completed_at') else None),
                              state=updates.get('state'),
                              error_message=updates.get('error_message')
                              )

    def delete_by_article_id(self, _id):
        lookup = {'item_id': _id}
        self.delete(lookup=lookup)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.publish.subscribers import SUBSCRIBER_TYPES

from test_factory import SuperdeskTestCase
from apps.publish import init_app
from apps.publish.formatters.aap_ipnews_formatter import AAPIpNewsFormatter
from apps.publish.formatters.aap_formatter_common import set_subject


class AapIpNewsFormatterTest(SuperdeskTestCase):
    subscribers = [{"_id": "1", "name": "ipnews", "subscriber_type": SUBSCRIBER_TYPES.WIRE, "media_type": "media",
                    "is_active": True, "sequence_num_settings": {"max": 10, "min": 1},
                    "destinations": [{"name": "AAP IPNEWS", "delivery_type": "email", "format": "AAP IPNEWS",
                                      "config": {"recipients": "test@sourcefabric.org"}
                                      }]
                    }]

    desks = [{'_id': 1, 'name': 'National'},
             {'_id': 2, 'name': 'Sports'},
             {'_id': 3, 'name': 'Finance'}]

    article = {
        'source': 'AAP',
        'anpa_category': [{'qcode': 'a'}],
        'headline': 'This is a test headline',
        'byline': 'joe',
        'slugline': 'slugline',
        'subject': [{'qcode': '02011001'}],
        'anpa_take_key': 'take_key',
        'unique_id': '1',
        'type': 'preformatted',
        'body_html': 'The story body',
        'word_count': '1',
        'priority': 1,
        'place': [{'qcode': 'VIC', 'name': 'VIC'}]
    }

    vocab = [{'_id': 'categories', 'items': [
        {'is_active': True, 'name': 'Overseas Sport', 'qcode': 'S', 'subject': '15000000'},
        {'is_active': True, 'name': 'Finance', 'qcode': 'F', 'subject': '04000000'}
    ]}, {'_id': 'geographical_restrictions', 'items': [{'name': 'New South Wales', 'value': 'NSW', 'is_active': True},
                                                       {'name': 'Victoria', 'value': 'VIC', 'is_active': True}]}]

    def setUp(self):
        super().setUp()
        self.app.data.insert('subscribers', self.subscribers)
        self.app.data.insert('vocabularies', self.vocab)
        self.app.data.insert('desks', self.desks)
        init_app(self.app)

    def testIPNewsFormatter(self):
        subscriber = self.app.data.find('subscribers', None, None)[0]

        f = AAPIpNewsFormatter()
        seq, item = f.format(self.article, subscriber)[0]

        self.assertGreater(int(seq), 0)
        self.assertEqual(seq, item['sequence'])
        item.pop('sequence')
        self.assertDictEqual(item,
                             {'category': 'a', 'texttab': 't', 'fullStory': 1, 'ident': '0',
                              'headline': 'VIC:This is a test headline', 'service_level': 'a', 'originator': 'AAP',
                              'take_key': 'take_key', 'article_text': 'The story body', 'priority': 'f', 'usn': '1',
                              'subject_matter': 'international law', 'news_item_type': 'News',
                              'subject_reference': '02011001', 'subject': 'crime, law and justice',
                              'wordcount': '1', 'subject_detail': 'international court or tribunal',
                              'genre': 'Current', 'keyword': 'slugline', 'author': 'joe'})

    def testIPNewsHtmlToText(self):
        article = {
            'source': 'AAP',
            'anpa_category': [{'qcode': 'a'}],
            'headline': 'This is a test headline',
            'byline': 'joe',
            'slugline': 'slugline',
            'subject': [{'qcode': '02011001'}],
            'anpa_take_key': 'take_key',
            'unique_id': '1',
            'type': 'text',
            'body_html': '<p>The story body line 1<br>Line 2</p>\
                         <p>abcdefghi abcdefghi abcdefghi abcdefghi abcdefghi abcdefghi abcdefghi abcdefghi more</p>',
            'word_count': '1',
            'priority': 1
        }

        subscriber = self.app.data.find('subscribers', None, None)[0]

        f = AAPIpNewsFormatter()
        seq, item = f.format(article, subscriber)[0]

        expected = '\r\nThe story body line 1 \r\nLine 2 \r\n\r\nabcdefghi abcdefghi abcdefghi abcdefghi ' \
                   'abcdefghi abcdefghi abcdefghi abcdefghi \r\nmore'
        self.assertEqual(item['article_text'], expected)

    def testMultipleCategories(self):
        article = {
            'source': 'AAP',
            'anpa_category': [{'name': 'Finance', 'qcode': 'F'},
                              {'name': 'Overseas Sport', 'qcode': 'S'}],
            'headline': 'This is a test headline',
            'byline': 'joe',
            'slugline': 'slugline',
            'subject': [{'qcode': '04001005'}, {'qcode': '15011002'}],
            'anpa_take_key': 'take_key',
            'unique_id': '1',
            'type': 'text',
            'body_html': 'body',
            'word_count': '1',
            'priority': 1,
            'task': {'desk': 1},
            'place': [{'qcode': 'VIC', 'name': 'VIC'}]
        }

        subscriber = self.app.data.find('subscribers', None, None)[0]

        f = AAPIpNewsFormatter()
        docs = f.format(article, subscriber)
        self.assertEqual(len(docs), 2)
        for seq, doc in docs:
            if doc['category'] == 'S':
                self.assertEqual(doc['subject_reference'], '15011002')
                self.assertEqual(doc['subject_detail'], 'four-man sled')
                self.assertEqual(doc['headline'], 'VIC:This is a test headline')
            if doc['category'] == 'F':
                self.assertEqual(doc['subject_reference'], '04001005')
                self.assertEqual(doc['subject_detail'], 'viniculture')
                self.assertEqual(doc['headline'], 'VIC:This is a test headline')
                codes = set(doc['selector_codes'].split(' '))
                expected_codes = set('cxx 0fh axx az and pxx 0ah 0ir 0px 0hw pnd pxd cnd cxd 0nl axd'.split(' '))
                self.assertSetEqual(codes, expected_codes)

    def testGeoBlock(self):
        article = {
            'source': 'AAP',
            'anpa_category': [{'qcode': 'a'}],
            'headline': 'This is a test headline',
            'byline': 'joe',
            'slugline': 'slugline',
            'subject': [{'qcode': '04001005'}, {'qcode': '15011002'}],
            'anpa_take_key': 'take_key',
            'unique_id': '1',
            'type': 'text',
            'body_html': 'body',
            'word_count': '1',
            'priority': 1,
            'task': {'desk': 1},
            'urgency': 1,
            'place': [{'qcode': 'VIC', 'name': 'VIC'}],
            'targeted_for': [{'name': 'New South Wales', 'allow': False}, {'name': 'Victoria', 'allow': True}]
        }

        subscriber = self.app.data.find('subscribers', None, None)[0]

        f = AAPIpNewsFormatter()
        seq, doc = f.format(article, subscriber)[0]
        codes = set(doc['selector_codes'].split(' '))
        expected_codes_str = 'an5 an4 an7 an6 ax5 an3 ax6 ax7 0hw an8 0ir px6 ax4 ax3 ax8 px5 0ah 0px px3 az'
        expected_codes_str += ' px8 0fh px7 px4 pn3 pn4 pn5 pn6 pn7 px0'
        expected_codes = set(expected_codes_str.split(' '))
        self.assertSetEqual(codes, expected_codes)

    def testGeoBlockNotTwoStates(self):
        article = {
            'source': 'AAP',
            'anpa_category': [{'qcode': 'a'}],
            'headline': 'This is a test headline',
            'byline': 'joe',
            'slugline': 'slugline',
            'subject': [{'qcode': '04001005'}, {'qcode': '15011002'}],
            'anpa_take_key': 'take_key',
            'unique_id': '1',
            'type': 'text',
            'body_html': 'body',
            'word_count': '1',
            'priority': 1,
            'task': {'desk': 1},
            'urgency': 1,
            'place': [{'qcode': 'VIC', 'name': 'VIC'}],
            'targeted_for': [{'name': 'New South Wales', 'allow': False}, {'name': 'Victoria', 'allow': False}]
        }

        subscriber = self.app.data.find('subscribers', None, None)[0]

        f = AAPIpNewsFormatter()
        seq, doc = f.format(article, subscriber)[0]
        codes = set(doc['selector_codes'].split(' '))
        expected_codes_str = 'an5 an4 an7 an6 ax5 ax6 ax7 an8 px6 ax4 ax8 px5 0ah 0px'
        expected_codes_str += ' px8 0fh px7 px4 pn4 pn5 pn6 pn7 px0'
        expected_codes = set(expected_codes_str.split(' '))
        self.assertSetEqual(codes, expected_codes)

    def testIpNewsFormatterNoSubject(self):
        article = {
            'source': 'AAP',
            'anpa_category': [{'qcode': 'a'}],
            'headline': 'This is a test headline',
            'byline': 'joe',
            'slugline': 'slugline',
            'subject': [],
            'anpa_take_key': 'take_key',
            'unique_id': '1',
            'type': 'text',
            'body_html': 'body',
            'word_count': '1',
            'priority': 1,
            'task': {'desk': 1},
            'urgency': 1,
            'place': [{'qcode': 'VIC', 'name': 'VIC'}]
        }
        subscriber = self.app.data.find('subscribers', None, None)[0]

        f = AAPIpNewsFormatter()
        seq, doc = f.format(article, subscriber)[0]
        self.assertEqual(doc['subject_reference'], '00000000')
        self.assertEqual(doc['headline'], 'VIC:This is a test headline')

        article = {
            'source': 'AAP',
            'anpa_category': [{'qcode': 'a'}],
            'headline': 'This is a test headline',
            'byline': 'joe',
            'slugline': 'slugline',
            'subject': None,
            'anpa_take_key': 'take_key',
            'unique_id': '1',
            'type': 'text',
            'body_html': 'body',
            'word_count': '1',
            'priority': 1,
            'task': {'desk': 1},
            'urgency': 1,
            'place': None
        }

        seq, doc = f.format(article, subscriber)[0]
        self.assertEqual(doc['subject_reference'], '00000000')
        self.assertEqual(doc['headline'], 'This is a test headline')

    def test_aap_ipnews_formatter_with_body_footer(self):
        subscriber = self.app.data.find('subscribers', None, None)[0]
        doc = self.article.copy()
        doc['body_footer'] = 'call helpline 999 if you are planning to quit smoking'

        f = AAPIpNewsFormatter()
        seq, item = f.format(doc, subscriber)[0]

        self.assertGreater(int(seq), 0)
        self.assertEqual(seq, item['sequence'])
        item.pop('sequence')
        self.assertDictEqual(item,
                             {'category': 'a', 'texttab': 't', 'fullStory': 1, 'ident': '0',
                              'headline': 'VIC:This is a test headline', 'service_level': 'a', 'originator': 'AAP',
                              'take_key': 'take_key',
                              'article_text': 'The story body<br>call helpline 999 if you are planning to quit smoking',
                              'priority': 'f', 'usn': '1',
                              'subject_matter': 'international law', 'news_item_type': 'News',
                              'subject_reference': '02011001', 'subject': 'crime, law and justice',
                              'wordcount': '1', 'subject_detail': 'international court or tribunal',
                              'genre': 'Current', 'keyword': 'slugline', 'author': 'joe'})


class DefaultSubjectTest(SuperdeskTestCase):

    def setUp(self):
        super().setUp()
        vocabularies = [{
            '_id': 'categories',
            'display_name': 'Categories',
            'type': 'manageable',
            'items': [
                {'is_active': True, 'name': 'Australian General News', 'qcode': 'a'},
                {'is_active': True, 'name': 'Australian Weather', 'qcode': 'b', 'subject': '17000000'},
                {'is_active': True, 'name': 'General Features', 'qcode': 'c'},
                {'is_active': False, 'name': 'Reserved (obsolete/unused)', 'qcode': 'd'},
                {'is_active': True, 'name': 'Entertainment', 'qcode': 'e', 'subject': '01000000'},
                {'is_active': True, 'name': 'Finance', 'qcode': 'f', 'subject': '04000000'},
                {'is_active': False, 'name': 'SportSet', 'qcode': 'g'},
                {'is_active': True, 'name': 'FormGuide', 'qcode': 'h'},
                {'is_active': True, 'name': 'International News', 'qcode': 'i'},
                {'is_active': False, 'name': 'Reserved (obsolete/unused)', 'qcode': 'k'},
                {'is_active': True, 'name': 'Press Release Service', 'qcode': 'j'},
                {'is_active': True, 'name': 'Lotteries', 'qcode': 'l'},
                {'is_active': True, 'name': 'Line Check Messages', 'qcode': 'm'},
                {'is_active': False, 'name': 'Reserved', 'qcode': 'n'},
                {'is_active': True, 'name': 'State Parliaments', 'qcode': 'o', 'subject': '11000000'},
                {'is_active': True, 'name': 'Federal Parliament', 'qcode': 'p', 'subject': '11000000'},
                {'is_active': True, 'name': 'Stockset', 'qcode': 'q', 'subject': '04000000'},
                {'is_active': True, 'name': 'Racing (Turf)', 'qcode': 'r', 'subject': '15000000'},
                {'is_active': True, 'name': 'Overseas Sport', 'qcode': 's', 'subject': '15000000'},
                {'is_active': True, 'name': 'Domestic Sport', 'qcode': 't', 'subject': '15000000'},
                {'is_active': False, 'name': 'Reserved (Obsolete/unused)', 'qcode': 'u'},
                {'is_active': True, 'name': 'Advisories', 'qcode': 'v'},
                {'is_active': False, 'name': 'Reserved (Obsolete/unused)', 'qcode': 'w'},
                {'is_active': True, 'name': 'Special Events (olympics/ Aus elections)', 'qcode': 'x'},
                {'is_active': False, 'name': 'Special Events (obsolete/unused)', 'qcode': 'y'},
                {'is_active': False, 'name': 'Supplementary Traffic', 'qcode': 'z'}
            ]
        }]

        self.app.data.insert('vocabularies', vocabularies)
        init_app(self.app)

    def test_subject(self):
        article = {
            'anpa_category': [{'qcode': 'a'}, {'qcode': 's'}],
            'subject': [{'qcode': '04001005'}, {'qcode': '15011002'}]
        }

        self.assertEqual(set_subject({'qcode': 'a'}, article), '04001005')
        self.assertEqual(set_subject({'qcode': 's'}, article), '15011002')
        article = {
            'anpa_category': [{'qcode': 'a'}, {'qcode': 's'}],
            'subject': None
        }

        self.assertEqual(set_subject({'qcode': 'a'}, article), None)
        self.assertEqual(set_subject({'qcode': 's'}, article), None)

        article = {
            'anpa_category': None,
            'subject': [{'qcode': '04001005'}, {'qcode': '15011002'}]
        }

        self.assertEqual(set_subject(None, article), '04001005')




import superdesk
from bs4 import BeautifulSoup
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE
from flask import render_template
from jinja2 import Template


def get_template(highlightId):
    """Return the string template associated with highlightId or none """
    if not highlightId:
        return None
    highlightService = superdesk.get_resource_service('highlights')
    highlight = highlightService.find_one(req=None, _id=highlightId)
    if not highlight or not highlight.get('template'):
        return None

    templateService = superdesk.get_resource_service('content_templates')
    template = templateService.find_one(req=None, _id=highlight.get('template'))
    if not template or 'body_html' not in template.get('data', {}):
        return None
    return template['data']['body_html']


class GenerateHighlightsService(superdesk.Service):
    def create(self, docs, **kwargs):
        """Generate highlights text item for given package.

        If doc.preview is True it won't save the item, only return.
        """
        service = superdesk.get_resource_service('archive')
        for doc in docs:
            preview = doc.get('preview', False)
            package = service.find_one(req=None, _id=doc['package'])
            if not package:
                superdesk.abort(404)
            stringTemplate = get_template(package.get('highlight'))

            doc.clear()
            doc[ITEM_TYPE] = CONTENT_TYPE.TEXT
            doc['headline'] = package.get('headline')
            doc['slugline'] = package.get('slugline')
            doc['byline'] = package.get('byline')
            doc['task'] = package.get('task')
            doc['family_id'] = package.get('guid')

            items = []
            for group in package.get('groups', []):
                for ref in group.get('refs', []):
                    if 'residRef' in ref:
                        item = service.find_one(req=None, _id=ref.get('residRef'))
                        if item:
                            html = item.get('body_html')
                            if html:
                                soup = BeautifulSoup(html, "html.parser")
                                item['first_paragraph_body_html'] = str(soup.p)
                            items.append(item)

            if stringTemplate:
                template = Template(stringTemplate)
                doc['body_html'] = template.render(package=package, items=items)
            else:
                doc['body_html'] = render_template('default_highlight_template.txt', package=package, items=items)
        if preview:
            return ['' for doc in docs]
        else:
            return service.post(docs, **kwargs)


class GenerateHighlightsResource(superdesk.Resource):
    """Generate highlights item for given package."""

    schema = {
        'package': {
            # not setting relation here, we will fetch it anyhow
            'type': 'string',
            'required': True,
        },
        'preview': {
            'type': 'boolean',
            'default': False,
        }
    }

    resource_methods = ['POST']
    item_methods = []
    privileges = {'POST': 'highlights'}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import superdesk
from eve.utils import config
from superdesk.publish.formatters import Formatter
from superdesk.errors import FormatterError
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, EMBARGO
from superdesk.metadata.packages import RESIDREF, GROUP_ID, GROUPS, ROOT_GROUP, REFS
from superdesk.utils import json_serialize_datetime_objectId


def filter_empty_vals(data):
    """Filter out `None` values from a given dict."""
    return dict(filter(lambda x: x[1], data.items()))


def format_cv_item(item):
    """Format item from controlled vocabulary for output."""
    return filter_empty_vals({
        'code': item.get('qcode'),
        'name': item.get('name'),
    })


class NINJSFormatter(Formatter):
    """
    NINJS Formatter
    """
    direct_copy_properties = ('versioncreated', 'usageterms', 'language', 'headline',
                              'urgency', 'pubstatus', 'mimetype', 'place', 'copyrightholder',
                              'body_text', 'body_html', 'profile', 'slugline', 'keywords')

    rendition_properties = ('href', 'width', 'height', 'mimetype')

    def format(self, article, subscriber):
        try:
            pub_seq_num = superdesk.get_resource_service('subscribers').generate_sequence_number(subscriber)

            ninjs = {
                '_id': article['_id'],
                'version': str(article.get(config.VERSION, 1)),
                'type': self._get_type(article)
            }

            try:
                ninjs['byline'] = self._get_byline(article)
            except:
                pass

            located = article.get('dateline', {}).get('located', {})
            if located:
                ninjs['located'] = located.get('city', '')

            for copy_property in self.direct_copy_properties:
                if article.get(copy_property) is not None:
                    ninjs[copy_property] = article[copy_property]

            if article.get('body_html'):
                ninjs['body_html'] = self.append_body_footer(article)

            if article.get('description'):
                ninjs['description_html'] = self.append_body_footer(article)

            if article[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
                ninjs['associations'] = self._get_associations(article)
            elif article.get('associations', {}):
                ninjs['associations'] = self._format_related(article, subscriber)

            if article.get(EMBARGO):
                ninjs['embargoed'] = article.get(EMBARGO).isoformat()

            if article.get('priority'):
                ninjs['priority'] = article['priority']
            else:
                ninjs['priority'] = 5

            if article.get('subject'):
                ninjs['subject'] = self._get_subject(article)

            if article.get('anpa_category'):
                ninjs['service'] = self._get_service(article)

            if article.get('renditions'):
                ninjs['renditions'] = self._get_renditions(article)

            if article.get('abstract'):
                ninjs['description_text'] = article.get('abstract')
            elif article.get('description_text'):
                ninjs['description_text'] = article.get('description_text')

            if article.get('company_codes'):
                ninjs['organisation'] = [{'name': c.get('name', ''), 'rel': 'Securities Identifier',
                                          'symbols': [{'ticker': c.get('qcode', ''),
                                                       'exchange': c.get('security_exchange', '')}]}
                                         for c in article['company_codes']]

            return [(pub_seq_num, json.dumps(ninjs, default=json_serialize_datetime_objectId))]
        except Exception as ex:
            raise FormatterError.ninjsFormatterError(ex, subscriber)

    def can_format(self, format_type, article):
        return format_type == 'ninjs'

    def _get_byline(self, article):
        if 'byline' in article:
            return article['byline'] or ''
        user = superdesk.get_resource_service('users').find_one(req=None, _id=article['original_creator'])
        if user:
            return user['display_name'] or ''
        raise Exception('User not found')

    def _get_type(self, article):
        if article[ITEM_TYPE] == CONTENT_TYPE.PREFORMATTED:
            return CONTENT_TYPE.TEXT
        return article[ITEM_TYPE]

    def _get_associations(self, article):
        """Create associations dict for package groups."""
        associations = dict()
        for group in article[GROUPS]:
            if group[GROUP_ID] == ROOT_GROUP:
                continue

            for ref in group[REFS]:
                if RESIDREF in ref:
                    items = associations.get(group[GROUP_ID], [])
                    item = {}
                    item['_id'] = ref[RESIDREF]
                    item[ITEM_TYPE] = ref[ITEM_TYPE]
                    items.append(item)
                    associations[group[GROUP_ID]] = items
        return associations

    def _format_related(self, article, subscriber):
        """Format all associated items for simple items (not packages)."""
        associations = {}
        for key, item in article.get('associations', {}).items():
            seq, formatted = self.format(item, subscriber)[0]
            associations[key] = json.loads(formatted)
        return associations

    def _get_subject(self, article):
        """Get subject list for article."""
        return [format_cv_item(item) for item in article.get('subject', [])]

    def _get_service(self, article):
        """Get service list for article.

        It's using `anpa_category` to populate service field for now.
        """
        return [format_cv_item(item) for item in article.get('anpa_category', [])]

    def _get_renditions(self, article):
        """Get renditions for article."""
        renditions = {}
        for name, rendition in article.get('renditions', {}).items():
            renditions[name] = self._format_rendition(rendition)
        return renditions

    def _format_rendition(self, rendition):
        """Format single rendition using fields whitelist."""
        return {field: rendition[field] for field in self.rendition_properties if field in rendition}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from . import currency_base


USD_TO_AUD = 1.40  # backup


def get_rate():
    """Get USD to AUD rate."""
    try:
        return currency_base.get_rate('USD', 'AUD')
    except:
        return USD_TO_AUD


def usd_to_aud(item, **kwargs):
    """Convert USD to AUD."""

    rate = get_rate()
    if os.environ.get('BEHAVE_TESTING'):
        rate = USD_TO_AUD

    return currency_base.do_conversion(item, rate, 'AUD %d', '\$([0-9]+)')


name = 'usd_to_aud'
label = 'Convert USD to AUD'
shortcut = 'c'
callback = usd_to_aud



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from test_factory import SuperdeskTestCase
from superdesk.utc import utcnow
from datetime import timedelta
from nose.tools import assert_raises
from superdesk.errors import PublishQueueError
from superdesk.publish.publish_content import is_on_time, update_content_state, get_queue_items
from apps.publish import init_app
from superdesk import config


class PublishContentTests(SuperdeskTestCase):
    queue_items = [{"_id": 1,
                    "destination": {
                        "delivery_type": "ftp",
                        "config": {},
                        "name": "destination1"
                    },
                    "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
                    "subscriber_id": "552ba73f1d41c8437971613e",
                    "state": "pending",
                    "_created": "2015-04-17T13:15:20.000Z",
                    "_updated": "2015-04-20T05:04:25.000Z",
                    "item_id": 1
                    },
                   {
                       "_id": 2,
                       "destination": {
                           "delivery_type": "ftp",
                           "config": {},
                           "name": "destination1"
                       },
                       "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
                       "subscriber_id": "552ba73f1d41c8437971613e",
                       "state": "pending",
                       "_created": "2015-04-17T13:15:20.000Z",
                       "_updated": "2015-04-20T05:04:25.000Z",
                       "item_id": 1,
                       "publish_schedule": utcnow() + timedelta(minutes=10)},
                   {
                       "_id": 3,
                       "destination": {
                           "delivery_type": "ftp",
                           "config": {},
                           "name": "destination1"
                       },
                       "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
                       "subscriber_id": "552ba73f1d41c8437971613e",
                       "state": "pending",
                       "_created": "2015-04-17T13:15:20.000Z",
                       "_updated": "2015-04-20T05:04:25.000Z",
                       "item_id": '2',
                       "publish_schedule": "2015-04-20T05:04:25.000Z"},
                   {
                       "_id": 4,
                       "destination": {
                           "delivery_type": "pull",
                           "config": {},
                           "name": "destination1"
                       },
                       "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
                       "subscriber_id": "552ba73f1d41c8437971613e",
                       "state": "pending",
                       "_created": "2015-04-17T13:15:20.000Z",
                       "_updated": "2015-04-20T05:04:25.000Z",
                       "item_id": '2'}]

    articles = [{'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a974-xy4532fe33f9',
                 '_id': '2',
                 'type': 'text',
                 'last_version': 3,
                 '_etag': '821739912837',
                 'body_html': 'Test body of the second article',
                 'urgency': 4,
                 'headline': 'Another two students missing',
                 'pubstatus': 'usable',
                 'firstcreated': utcnow(),
                 'byline': 'By Alan Karben',
                 'ednote': 'Andrew Marwood contributed to this article',
                 'dateline': {'text': 'Sydney'},
                 'keywords': ['Student', 'Crime', 'Police', 'Missing'],
                 'subject':[{'qcode': '17004000', 'name': 'Statistics'},
                            {'qcode': '04001002', 'name': 'Weather'}],
                 'expiry': utcnow() + timedelta(minutes=20),
                 'publish_schedule': "2016-05-30T10:00:00+0000",
                 'state': 'scheduled',
                 'original_creator': '553cea4d1d41c85d4d42ff98',
                 'task': {'desk': 1}}]

    desks = [{'_id': 1, 'content_expiry': 20, 'incoming_stage': 1}]
    stages = [{'_id': 1, 'desk': 1}]
    published = [{'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a974-xy4532fe33f9',
                  'item_id': '2',
                  '_id': 333,
                  '_etag': 821739912837,
                  'last_version': 3,
                  'body_html': 'Test body of the second article',
                  'urgency': 4,
                  'headline': 'Another two students missing',
                  'pubstatus': 'usable',
                  'firstcreated': utcnow(),
                  'byline': 'By Alan Karben',
                  'ednote': 'Andrew Marwood contributed to this article',
                  'dateline': {'text': 'Sydney'},
                  'keywords': ['Student', 'Crime', 'Police', 'Missing'],
                  'subject':[{'qcode': '17004000', 'name': 'Statistics'},
                             {'qcode': '04001002', 'name': 'Weather'}],
                  'expiry': utcnow() + timedelta(minutes=20),
                  'publish_schedule': "2016-05-30T10:00:00+0000",
                  'state': 'scheduled'}]

    def setUp(self):
        super().setUp()
        with self.app.app_context():
            self.app.data.insert('stages', self.stages)
            self.app.data.insert('desks', self.desks)
            self.app.data.insert('archive', self.articles)
            self.app.data.insert('published', self.published)
            init_app(self.app)

    def test_scheduled_items(self):
        self.assertTrue(is_on_time(self.queue_items[0]))
        self.assertFalse(is_on_time(self.queue_items[1]))
        with self.app.app_context():
            with assert_raises(PublishQueueError):
                self.assertTrue(is_on_time(self.queue_items[2]))

    def test_update_content_state(self):
        with self.app.app_context():
            published_items = self.app.data.find('published', None, None)
            archive_items = self.app.data.find_all('archive', None)
            self.assertEquals(1, published_items.count())
            self.assertEquals(1, archive_items.count())
            self.assertEquals(published_items[0]['state'], 'scheduled')
            self.assertEquals(archive_items[0]['state'], 'scheduled')
            update_content_state(self.queue_items[2])
            published_items = self.app.data.find('published', None, None)
            archive_items = self.app.data.find_all('archive', None)
            self.assertEquals(1, published_items.count())
            self.assertEquals(1, archive_items.count())
            self.assertEquals(published_items[0]['state'], 'published')
            self.assertEquals(archive_items[0]['state'], 'published')

    def test_queue_items(self):
        with self.app.app_context():
            self.app.data.insert('publish_queue', self.queue_items)
            items = get_queue_items()
            self.assertEqual(3, items.count())
            ids = [item[config.ID_FIELD] for item in items]
            self.assertNotIn(4, ids)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk
from cerberus import Validator
from superdesk.metadata.item import ITEM_TYPE

logger = logging.getLogger(__name__)


class ValidateResource(superdesk.Resource):
    schema = {
        'act': {'type': 'string', 'required': True},
        'type': {'type': 'string', 'required': True},
        'validate': {
            'type': 'dict',
            'required': True
        }
    }

    resource_methods = ['POST']
    item_methods = []


class ValidateService(superdesk.Service):

    def create(self, docs, **kwargs):
        for doc in docs:
            doc['errors'] = self._validate(doc, **kwargs)

        return [doc['errors'] for doc in docs]

    def _validate(self, doc, **kwargs):
        lookup = {'act': doc['act'], 'type': doc[ITEM_TYPE]}
        use_headline = kwargs and 'headline' in kwargs
        validators = superdesk.get_resource_service('validators').get(req=None, lookup=lookup)
        for validator in validators:
            v = Validator()
            v.allow_unknown = True
            v.validate(doc['validate'], validator['schema'])
            error_list = v.errors
            response = []
            for e in error_list:
                if error_list[e] == 'required field' or type(error_list[e]) is dict:
                    message = '{} is a required field'.format(e.upper())
                elif 'min length is' in error_list[e]:
                    message = '{} is too short'.format(e.upper())
                elif 'max length is' in error_list[e]:
                    message = '{} is too long'.format(e.upper())
                else:
                    message = '{} {}'.format(e.upper(), error_list[e])

                if use_headline:
                    response.append('{}: {}'.format(doc['validate'].get('headline',
                                                                        doc['validate'].get('_id')), message))
                else:
                    response.append(message)
            return response
        else:
            return ['validator was not found for {}'.format(doc['act'])]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

'''
Utilities for extractid metadata from image files.
'''

from PIL import Image, ExifTags
from flask import json


def get_meta(file_stream):
    '''
    Returns the image metadata in a dictionary of tag:value pairs.

    @param file_stream: stream
    '''
    current = file_stream.tell()
    file_stream.seek(0)
    img = Image.open(file_stream)
    if not hasattr(img, '_getexif'):
        return {}
    rv = img._getexif()
    if not rv:
        return {}
    exif = dict(rv)
    file_stream.seek(current)

    exif_meta = {}
    for k, v in exif.items():
        try:
            json.dumps(v)
            key = ExifTags.TAGS[k].strip()

            if key == 'GPSInfo':
                # lookup GPSInfo description key names
                value = {ExifTags.GPSTAGS[vk].strip(): vv for vk, vv in v.items()}
                exif_meta[key] = value
            else:
                value = v.decode('UTF-8') if isinstance(v, bytes) else v
                exif_meta[key] = value
        except:
            # ignore fields we can't store in db
            pass
    # Remove this as it's too long to send in headers
    exif_meta.pop('UserComment', None)

    return exif_meta



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
import flask
import importlib
import jinja2
import eve
import superdesk
from flask.ext.mail import Mail
from eve.io.mongo import MongoJSONEncoder
from eve.render import send_response
from superdesk.celery_app import init_celery
from eve.auth import TokenAuth
from superdesk.storage.desk_media_storage import SuperdeskGridFSMediaStorage
from superdesk.validator import SuperdeskValidator
from raven.contrib.flask import Sentry
from superdesk.errors import SuperdeskError, SuperdeskApiError
from superdesk.io import registered_feeding_services
from superdesk.datalayer import SuperdeskDataLayer  # noqa


sentry = Sentry(register_signal=False, wrap_wsgi=False)


def get_app(config=None, media_storage=None):
    """App factory.

    :param config: configuration that can override config from `default_settings.py`
    :return: a new SuperdeskEve app instance
    """

    abs_path = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    app_config = flask.Config(abs_path)
    app_config.from_object('superdesk.factory.default_settings')
    app_config.from_object(config or {})
    app_config.setdefault('APP_ABSPATH', abs_path)
    app_config.setdefault('DOMAIN', {})

    if media_storage is None:
        media_storage = SuperdeskGridFSMediaStorage

    app = eve.Eve(
        data=SuperdeskDataLayer,
        auth=TokenAuth,
        media=media_storage,
        settings=app_config,
        json_encoder=MongoJSONEncoder,
        validator=SuperdeskValidator,
        template_folder=os.path.join(abs_path, 'templates'))

    superdesk.app = app

    custom_loader = jinja2.ChoiceLoader([
        app.jinja_loader,
        jinja2.FileSystemLoader([abs_path + '/../templates'])
    ])
    app.jinja_loader = custom_loader

    app.mail = Mail(app)

    @app.errorhandler(SuperdeskError)
    def client_error_handler(error):
        """Return json error response.

        :param error: an instance of :attr:`superdesk.SuperdeskError` class
        """
        return send_response(None, (error.to_dict(), None, None, error.status_code))

    @app.errorhandler(500)
    def server_error_handler(error):
        """Log server errors."""
        app.sentry.captureException()
        superdesk.logger.exception(error)
        return_error = SuperdeskApiError.internalError()
        return client_error_handler(return_error)

    init_celery(app)

    for module_name in app.config['INSTALLED_APPS']:
        app_module = importlib.import_module(module_name)
        try:
            app_module.init_app(app)
        except AttributeError:
            pass

    for resource in superdesk.DOMAIN:
        app.register_resource(resource, superdesk.DOMAIN[resource])

    for blueprint in superdesk.BLUEPRINTS:
        prefix = app.api_prefix or None
        app.register_blueprint(blueprint, url_prefix=prefix)

    for name, jinja_filter in superdesk.JINJA_FILTERS.items():
        app.jinja_env.filters[name] = jinja_filter

    # we can only put mapping when all resources are registered
    app.data.elastic.put_mapping(app)

    app.sentry = sentry
    sentry.init_app(app)

    # instantiate registered provider classes (leave non-classes intact)
    for key, provider in registered_feeding_services.items():
        registered_feeding_services[key] = provider() if isinstance(provider, type) else provider

    return app



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re


def find_and_replace(item, **kwargs):
    """Finds the instances of '$' character and
    replaces with 'AUD ' (with a trailing space)"""

    aud = 'AUD '

    for field in ['body_html', 'body_text', 'abstract', 'headline', 'slugline']:
        if item.get(field, None):
            item[field] = re.sub('[$]', aud, item[field])

    return item

name = 'dollar_AUD_character_replace'
label = '$ -> AUD'
shortcut = '$'
callback = find_and_replace



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .commands import LegalArchiveImport
from test_factory import SuperdeskTestCase


class LegalArchiveTestCase(SuperdeskTestCase):

    desks = [{'_id': '123', 'name': 'Sports'}]
    users = [{'_id': '123', 'username': 'test1', 'first_name': 'test', 'last_name': 'user', 'email': 'a@a.com'}]
    stages = [{'_id': '123', 'name': 'working stage', 'desk': '123'}]
    archive = [
        {
            'task': {'desk': '123', 'stage': '123'}
        },
        {
            'task': {'desk': '1234', 'stage': None, 'user': '123'}
        },
        {
            'task': {'desk': '1234', 'stage': 'dddd', 'user': 'test'}
        }
    ]

    def setUp(self):
        super().setUp()
        self.app.data.insert("desks", self.desks)
        self.app.data.insert("users", self.users)
        self.app.data.insert("stages", self.stages)

    def test_denormalize_desk_user(self):
        LegalArchiveImport()._denormalize_user_desk(self.archive[0], '')
        task = self.archive[0]['task']
        self.assertEqual(task.get('desk'), 'Sports')
        self.assertEqual(task.get('stage'), 'working stage')
        self.assertEqual(task.get('user'), '')

    def test_denormalize_not_configured_desk(self):
        LegalArchiveImport()._denormalize_user_desk(self.archive[1], '')
        task = self.archive[1]['task']
        self.assertEqual(task.get('desk'), '1234')
        self.assertEqual(task.get('stage'), None)
        self.assertEqual(task.get('user'), 'test user')

    def test_denormalize_not_configured_desk_stage_user(self):
        LegalArchiveImport()._denormalize_user_desk(self.archive[2], '')
        task = self.archive[2]['task']
        self.assertEqual(task.get('desk'), '1234')
        self.assertEqual(task.get('stage'), 'dddd')
        self.assertEqual(task.get('user'), '')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.publish.subscribers import SUBSCRIBER_TYPES

from test_factory import SuperdeskTestCase
from apps.publish import init_app
from apps.publish.formatters.aap_bulletinbuilder_formatter import AAPBulletinBuilderFormatter
from superdesk.utils import json_serialize_datetime_objectId
from superdesk import json
from superdesk.utc import utcnow
from bson import ObjectId


class AapBulletinBuilderFormatterTest(SuperdeskTestCase):
    subscribers = [{"_id": "1", "name": "Test", "subscriber_type": SUBSCRIBER_TYPES.WIRE, "media_type": "media",
                    "is_active": True, "sequence_num_settings": {"max": 10, "min": 1},
                    "destinations": [{"name": "AAP Bulletin Builder", "delivery_type": "pull",
                                      "format": "AAP BULLETIN BUILDER"
                                      }]
                    }]

    def setUp(self):
        super().setUp()
        self.app.data.insert('subscribers', self.subscribers)
        init_app(self.app)
        self._formatter = AAPBulletinBuilderFormatter()

    def test_bulletin_builder_formatter(self):
        article = {
            'source': 'AAP',
            'anpa_category': [{'qcode': 'a'}],
            'headline': 'This is a test headline',
            'byline': 'joe',
            'slugline': 'slugline',
            'subject': [{'qcode': '02011001'}],
            'anpa_take_key': 'take_key',
            'unique_id': '1',
            'type': 'preformatted',
            'body_html': 'The story body',
            'word_count': '1',
            'priority': '1',
            'firstcreated': utcnow(),
            'versioncreated': utcnow(),
            'lock_user': ObjectId()
        }

        subscriber = self.app.data.find('subscribers', None, None)[0]
        seq, item = self._formatter.format(article, subscriber)[0]
        self.assertGreater(int(seq), 0)
        self.assertEqual(json.dumps(article, default=json_serialize_datetime_objectId), item)

    def test_strip_html(self):
        article = {
            'source': 'AAP',
            'headline': 'This is a test headline',
            'type': 'text',
            'body_html': ('<p>The story body line 1<br>Line 2</p>'
                          '<p>abcdefghi abcdefghi abcdefghi abcdefghi abcdefghi'
                          '<span> abcdefghi</span> abcdefghi abcdefghi more</p>'
                          '<table><tr><td>test</td></tr></table>')
        }

        body_text = ('The story body line 1 Line 2\r\n\r\n'
                     'abcdefghi abcdefghi abcdefghi abcdefghi abcdefghi'
                     ' abcdefghi abcdefghi abcdefghi more\r\n\r\n'
                     'test')

        subscriber = self.app.data.find('subscribers', None, None)[0]
        seq, item = self._formatter.format(article, subscriber)[0]
        self.assertGreater(int(seq), 0)
        test_article = json.loads(item)
        self.assertEqual(test_article['body_text'], body_text)

    def test_locator(self):
        article = {
            'source': 'AAP',
            'anpa_category': [{'qcode': 's'}],
            'headline': 'This is a test headline',
            'byline': 'joe',
            'slugline': 'slugline',
            'subject': [{'qcode': '15017000'}],
            'anpa_take_key': 'take_key',
            'unique_id': '1',
            'type': 'preformatted',
            'body_html': 'The story body',
            'word_count': '1',
            'priority': '1',
            'firstcreated': utcnow(),
            'versioncreated': utcnow(),
            'lock_user': ObjectId(),
            'place': [{'qcode': 'VIC', 'name': 'VIC'}]
        }

        subscriber = self.app.data.find('subscribers', None, None)[0]
        seq, item = self._formatter.format(article, subscriber)[0]
        self.assertGreater(int(seq), 0)
        test_article = json.loads(item)
        self.assertEqual(test_article['headline'], 'This is a test headline')
        self.assertEqual(test_article['place'][0]['qcode'], 'CRIK')
        article['anpa_category'] = [{'qcode': 'a'}]
        article['place'] = [{'qcode': 'VIC', 'name': 'VIC'}]
        seq, item = self._formatter.format(article, subscriber)[0]
        self.assertGreater(int(seq), 0)
        test_article = json.loads(item)
        self.assertEqual(test_article['headline'], 'This is a test headline')
        self.assertEqual(test_article['place'][0]['qcode'], 'VIC')

    def test_body_footer(self):
        article = {
            'source': 'AAP',
            'anpa_category': [{'qcode': 's'}],
            'headline': 'This is a test headline',
            'byline': 'joe',
            'slugline': 'slugline',
            'subject': [{'qcode': '15017000'}],
            'anpa_take_key': 'take_key',
            'unique_id': '1',
            'type': 'preformatted',
            'body_html': 'The story body',
            'word_count': '1',
            'priority': '1',
            'firstcreated': utcnow(),
            'versioncreated': utcnow(),
            'lock_user': ObjectId(),
            'body_footer': 'call helpline 999 if you are planning to quit smoking'
        }

        subscriber = self.app.data.find('subscribers', None, None)[0]
        seq, item = self._formatter.format(article, subscriber)[0]

        formatted_article = json.loads(item)
        self.assertEqual(formatted_article['body_text'],
                         'The story body call helpline 999 if you are planning to quit smoking')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from eve.versioning import resolve_document_version
from .common import BasePublishService, BasePublishResource, ITEM_KILL
from eve.utils import config
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE, GUID_FIELD, PUB_STATUS
from superdesk.metadata.packages import PACKAGE_TYPE
from superdesk import get_resource_service
from superdesk.publish import SUBSCRIBER_TYPES
from superdesk.utc import utcnow
import logging
from copy import copy
from superdesk.emails import send_article_killed_email
from superdesk.errors import SuperdeskApiError
from apps.archive.common import ITEM_OPERATION, ARCHIVE, insert_into_versions
from apps.templates.content_templates import render_content_template

logger = logging.getLogger(__name__)


class KillPublishResource(BasePublishResource):
    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, 'kill')


class KillPublishService(BasePublishService):
    publish_type = 'kill'
    published_state = 'killed'

    def __init__(self, datasource=None, backend=None):
        super().__init__(datasource=datasource, backend=backend)

    def on_update(self, updates, original):
        # check if we are trying to kill and item that is contained in normal non takes package
        # and the package itself is not killed.

        packages = self.package_service.get_packages(original[config.ID_FIELD])
        if packages and packages.count() > 0:
            for package in packages:
                if package[ITEM_STATE] != CONTENT_STATE.KILLED and package.get(PACKAGE_TYPE, '') == '':
                    raise SuperdeskApiError.badRequestError(message='This item is in a package. '
                                                                    'It needs to be removed '
                                                                    'before the item can be killed')

        updates['pubstatus'] = PUB_STATUS.CANCELED
        super().on_update(updates, original)
        updates[ITEM_OPERATION] = ITEM_KILL
        self.takes_package_service.process_killed_takes_package(original)
        get_resource_service('archive_broadcast').spike_item(original)

    def update(self, id, updates, original):
        """
        Kill will broadcast kill email notice to all subscriber in the system and then kill the item.
        If item is a take then all the takes are killed as well.
        """
        self.broadcast_kill_email(original)
        super().update(id, updates, original)
        self._publish_kill_for_takes(updates, original)
        get_resource_service('archive_broadcast').kill_broadcast(updates, original)

    def broadcast_kill_email(self, original):
        """
        Sends the broadcast email to all subscribers (including in-active subscribers)
        :param original: Document to kill
        """
        # Get all subscribers
        subscribers = list(get_resource_service('subscribers').get(req=None, lookup=None))
        recipients = [s.get('email') for s in subscribers if s.get('email')]
        # send kill email.
        send_article_killed_email(original, recipients, utcnow())

    def _publish_kill_for_takes(self, updates, original):
        """
        Kill all the takes in a takes package.
        :param updates: Updates of the original document
        :param original: Document to kill
        """
        package = self.takes_package_service.get_take_package(original)
        last_updated = updates.get(config.LAST_UPDATED, utcnow())
        if package:
            for ref in[ref for group in package.get('groups', []) if group['id'] == 'main'
                       for ref in group.get('refs')]:
                if ref[GUID_FIELD] != original[config.ID_FIELD]:
                    original_data = super().find_one(req=None, _id=ref[GUID_FIELD])
                    updates_data = copy(updates)
                    '''
                    We need to update the archive item and not worry about queued as we could have a takes only going
                    to digital client.
                    Popping out the config.VERSION as Take referenced by original and Take referenced by original_data
                    might have different and if not popped out then it might jump the versions.
                    '''
                    updates_data.pop(config.VERSION, None)
                    self._set_updates(original_data, updates_data, last_updated)
                    queued = self.publish(doc=original_data,
                                          updates=updates_data,
                                          target_media_type=SUBSCRIBER_TYPES.WIRE)
                    self._update_archive(original=original_data, updates=updates_data,
                                         should_insert_into_versions=True)
                    self.update_published_collection(published_item_id=original_data['_id'])

                    if not queued:
                        logger.error("Could not publish the kill for take {} with headline {}".
                                     format(original_data.get(config.ID_FIELD), original_data.get('headline')))

    def get_subscribers(self, doc, target_media_type):
        """
        Get the subscribers for this document based on the target_media_type for kill.
        Kill is sent to all subscribers that have received the item previously (published or corrected)
        :param doc: Document to kill
        :param target_media_type: dictate if the doc being queued is a Takes Package or an Individual Article.
                Valid values are - Wire, Digital. If Digital then the doc being queued is a Takes Package and if Wire
                then the doc being queued is an Individual Article.
        :return: (list, list) List of filtered subscribers,
                List of subscribers that have not received item previously (empty list in this case).
        """

        subscribers, subscribers_yet_to_receive = [], []
        query = {'$and': [{'item_id': doc[config.ID_FIELD]},
                          {'publishing_action': {'$in': [CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED]}}]}
        subscribers = self._get_subscribers_for_previously_sent_items(query)

        return subscribers, subscribers_yet_to_receive

    def kill_item(self, item):
        """
        Kill the item after applying the template.
        :param dict item: Item
        """
        # get the kill template
        template = get_resource_service('content_templates').get_template_by_name('kill')
        if not template:
            SuperdeskApiError.badRequestError(message="Kill Template missing.")

        # apply the kill template
        updates = render_content_template(item, template)
        # resolve the document version
        resolve_document_version(document=updates, resource=ARCHIVE, method='PATCH', latest_doc=item)
        # kill the item
        self.patch(item.get(config.ID_FIELD), updates)
        # insert into versions
        insert_into_versions(id_=item[config.ID_FIELD])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk
from superdesk.errors import BulkIndexError


class IndexFromMongo(superdesk.Command):
    """
    Index the specified mongo collection in the specified elastic collection/type.
    This will use the default APP mongo DB to read the data and the default Elastic APP index.
    """
    option_list = [
        superdesk.Option('--from', '-f', dest='mongo_collection_name', required=True),
        superdesk.Option('--page-size', '-p', dest='page_size')
    ]
    default_page_size = 500

    def run(self, mongo_collection_name, page_size):
        bucket_size = int(page_size) if page_size else self.default_page_size
        print('Indexing data from mongo/{} to elastic/{}'.format(mongo_collection_name, mongo_collection_name))

        service = superdesk.get_resource_service(mongo_collection_name)
        cursor = service.get_from_mongo(None, {})
        count = cursor.count()
        no_of_buckets = len(range(0, count, bucket_size))
        print('Number of items to index: {}, pages={}'.format(count, no_of_buckets))

        for x in range(0, no_of_buckets):
            skip = x * bucket_size
            print('Page : {}, skip: {}'.format(x + 1, skip))
            cursor = service.get_from_mongo(None, {})
            cursor.skip(skip)
            cursor.limit(bucket_size)
            items = list(cursor)
            print('Inserting {} items'.format(len(items)))
            success, failed = superdesk.app.data._search_backend(mongo_collection_name).bulk_insert(
                mongo_collection_name, items)
            print('Inserted {} items'.format(success))
            if failed:
                print('Failed to do bulk insert of items {}. Errors: {}'.format(len(failed), failed))
                raise BulkIndexError(resource=mongo_collection_name, errors=failed)

        return 'Finished indexing collection {}'.format(mongo_collection_name)


superdesk.command('app:index_from_mongo', IndexFromMongo())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from test_factory import SuperdeskTestCase
from apps.publish import init_app
from apps.publish.formatters.field_mappers.locator_mapper import LocatorMapper


class SelectorcodeMapperTest(SuperdeskTestCase):

    desks = [{'_id': 1, 'name': 'National'},
             {'_id': 2, 'name': 'Sports'},
             {'_id': 3, 'name': 'Finance'}]

    def setUp(self):
        super().setUp()
        self.app.data.insert('desks', self.desks)
        init_app(self.app)

    def test_map_subject_code(self):
        article1 = {
            'subject': [{'qcode': '10006000'}, {'qcode': '15011002'}]

        }
        article2 = {
            'subject': [{'qcode': '15011002'}, {'qcode': '10006000'}]

        }
        article3 = {
            'subject': [{'qcode': '15063000'}, {'qcode': '15067000'}]

        }
        article4 = {
            'subject': [{'qcode': '9999999'}]

        }
        category1 = 'I'
        category2 = 'D'
        f = LocatorMapper()
        self.assertEqual(f.map(article1, category1), 'TRAVI')
        self.assertEqual(f.map(article2, category2), 'TRAVD')
        self.assertEqual(f.map(article3, 'S'), 'TTEN')
        self.assertIsNone(f.map(article4, category2))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.locators.locators import find_cities


def dpa_derive_dateline(item, **kwargs):
    """
    DPA content is recieved in IPTC7901 format, this macro attempts to parse a dateline from the first few lines of
    the item body and populate the dataline location, it also populates the dateline source.
    If a dateline is matched the coresponding string is removed from the article text.
    :param item:
    :param kwargs:
    :return:
    """
    lines = item['body_html'].splitlines()
    if lines:
        # expect the dateline in the first 5 lines, sometimes there is what appears to be a headline preceeding it.
        for line_num in range(0, min(len(lines), 5)):
            city, source, the_rest = lines[line_num].partition(' (dpa) - ')
            # test if we found a candidate and ensure that the city starts the line and is not crazy long
            if source and lines[line_num].find(city) == 0 and len(city) < 20:
                cities = find_cities()
                located = [c for c in cities if c['city'].lower() == city.lower()]
                if 'dateline' not in item:
                    item['dateline'] = {}
                item['dateline']['located'] = located[0] if len(located) > 0 else {'city_code': city, 'city': city,

                                                                                   'tz': 'UTC', 'dateline': 'city'}
                item['dateline']['source'] = 'dpa'
                item['dateline']['text'] = city
                lines[line_num] = lines[line_num].replace(city + source, '')
                item['body_html'] = '\r\n'.join(lines)
                break
    return item


name = 'Derive dateline from article text for DPA'
label = 'DPA dateline derivation'
shortcut = '$'
callback = dpa_derive_dateline



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.publish.formatters import Formatter
import superdesk
from superdesk.utils import json_serialize_datetime_objectId
from superdesk.errors import FormatterError
from bs4 import BeautifulSoup
from .field_mappers.locator_mapper import LocatorMapper


class AAPBulletinBuilderFormatter(Formatter):
    """
    Bulletin Builder Formatter
    """
    def format(self, article, subscriber):
        """
        Formats the article as require by the subscriber
        :param dict article: article to be formatted
        :param dict subscriber: subscriber receiving the article
        :return: tuple (int, str) of publish sequence of the subscriber, formatted article as string
        """
        try:
            article['slugline'] = self.append_legal(article=article, truncate=True)
            pub_seq_num = superdesk.get_resource_service('subscribers').generate_sequence_number(subscriber)
            body_html = self.append_body_footer(article).strip('\r\n')
            soup = BeautifulSoup(body_html, "html.parser")
            for br in soup.find_all('br'):
                # remove the <br> tag
                br.replace_with(' {}'.format(br.get_text()))

            for p in soup.find_all('p'):
                # replace <p> tag with two carriage return
                p.replace_with('{}\r\n\r\n'.format(p.get_text()))

            article['body_text'] = soup.get_text()

            # get the first category and derive the locator
            category = next((iter(article.get('anpa_category', []))), None)
            if category:
                locator = LocatorMapper().map(article, category.get('qcode').upper())
                if locator:
                    article['place'] = [{'qcode': locator, 'name': locator}]

            return [(pub_seq_num, superdesk.json.dumps(article, default=json_serialize_datetime_objectId))]
        except Exception as ex:
            raise FormatterError.bulletinBuilderFormatterError(ex, subscriber)

    def can_format(self, format_type, article):
        return format_type == 'AAP BULLETIN BUILDER'



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.locators.locators import find_cities
from bs4 import BeautifulSoup
import logging
from apps.archive.common import format_dateline_to_locmmmddsrc
from superdesk.utc import get_date
from superdesk.metadata.item import BYLINE

logger = logging.getLogger(__name__)


def reuters_derive_dateline(item, **kwargs):
    """
    It seems that most locations injected into the item by the parser are Bangalor
    This function looks for a dateline in the article body an uses that.
    :param items:
    :return:
    """
    try:
        html = item.get('body_html')
        if html:
            soup = BeautifulSoup(html, "html.parser")
            pars = soup.findAll('p')
            if len(pars) >= 2:
                if BYLINE in item and item.get(BYLINE) in pars[0].get_text():
                    first = pars[1].get_text()
                else:
                    first = pars[0].get_text()
                city, source, the_rest = first.partition(' (Reuters) - ')
                if source:
                    # sometimes the city is followed by a comma and either a date or a state
                    city = city.split(',')[0]
                    if any(char.isdigit() for char in city):
                        return
                    cities = find_cities()
                    located = [c for c in cities if c['city'].lower() == city.lower()]
                    # if not dateline we create one
                    if 'dateline' not in item:
                        item['dateline'] = {}
                    # there is already a dateline that is not Bangalore don't do anything just return
                    elif 'located' in item['dateline'] and 'BANGALORE' != item['dateline']['located'].get(
                            'city').upper():
                        return

                    item['dateline']['located'] = located[0] if len(located) > 0 else {'city_code': city,
                                                                                       'city': city,
                                                                                       'tz': 'UTC',
                                                                                       'dateline': 'city'}
                    item['dateline']['source'] = item.get('original_source', 'Reuters')
                    item['dateline']['text'] = format_dateline_to_locmmmddsrc(item['dateline']['located'],
                                                                              get_date(item['firstcreated']),
                                                                              source=item.get('original_source',
                                                                                              'Reuters'))

        return item
    except:
        logging.exception('Reuters dateline macro exception')


name = 'Derive dateline from article text for Reuters'
label = 'Reuters dateline derivation'
shortcut = '$'
callback = reuters_derive_dateline



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from datetime import datetime
import dateutil.parser
from superdesk.io import register_feed_parser
from superdesk.io.feed_parsers import XMLFeedParser
import xml.etree.ElementTree as etree
from superdesk.errors import ParserError
from superdesk.utc import utc
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE
from superdesk.etree import get_word_count
from superdesk.io.iptc import subject_codes

SUBJECT_TYPE = 'tobject.subject.type'
SUBJECT_MATTER = 'tobject.subject.matter'
SUBJECT_DETAIL = 'tobject.subject.detail'

subject_fields = (SUBJECT_TYPE, SUBJECT_MATTER, SUBJECT_DETAIL)


def get_places(docdata):
    places = []
    evloc = docdata.find('evloc')
    if evloc is not None:
        places.append({
            'name': evloc.attrib.get('city'),
            'code': evloc.attrib.get('iso-cc'),
        })
    return places


def get_subjects(tree):
    """
    Finds all the subject tags in the passed tree and returns the parsed subjects. All entries will have both the
    name and qcode populated.
    :param tree:
    :return: a list of subject dictionaries
    """
    subjects = []
    for elem in tree.findall('head/tobject/tobject.subject'):
        qcode = elem.get('tobject.subject.refnum')
        for field in subject_fields:
            if elem.get(field):
                if field == SUBJECT_TYPE:
                    field_qcode = qcode[:2] + '000000'
                elif field == SUBJECT_MATTER:
                    field_qcode = qcode[:5] + '000'
                else:
                    field_qcode = qcode

                if subject_codes.get(field_qcode):
                    subjects.append({
                        'name': elem.get(field),
                        'qcode': field_qcode
                    })
        if not any(c['qcode'] == qcode for c in subjects) and subject_codes.get(qcode):
            subjects.append({'name': subject_codes[qcode], 'qcode': qcode})
    return subjects


def get_keywords(docdata):
    return [keyword.attrib['key'] for keyword in docdata.findall('key-list/keyword')]


def get_content(tree):
    elements = []
    for elem in tree.find('body/body.content'):
        elements.append(etree.tostring(elem, encoding='unicode'))
    return ''.join(elements)


def get_norm_datetime(tree):
    if tree is None:
        return

    try:
        value = datetime.strptime(tree.attrib['norm'], '%Y%m%dT%H%M%S')
    except ValueError:
        try:
            value = datetime.strptime(tree.attrib['norm'], '%Y%m%dT%H%M%S%z')
        except ValueError:
            value = dateutil.parser.parse(tree.attrib['norm'])

    return utc.normalize(value) if value.tzinfo else value


def get_byline(tree):
    elem = tree.find('body/body.head/byline')
    byline = ''
    if elem is not None:
        byline = elem.text
        person = elem.find('person')
        if person is not None:
            byline = "{} {}".format(byline.strip(), person.text.strip())
    return byline


def parse_meta(tree, item):
    for elem in tree.findall('head/meta'):
        attribute_name = elem.get('name')

        if attribute_name == 'anpa-keyword':
            item['slugline'] = elem.get('content')
        elif attribute_name == 'anpa-sequence':
            item['ingest_provider_sequence'] = elem.get('content')
        elif attribute_name == 'anpa-category':
            item['anpa_category'] = [{'qcode': elem.get('content'), 'name': ''}]
        elif attribute_name == 'anpa-wordcount':
            item['word_count'] = int(elem.get('content'))
        elif attribute_name == 'anpa-takekey':
            item['anpa_take_key'] = elem.get('content')
        elif attribute_name == 'anpa-format':
            anpa_format = elem.get('content').lower() if elem.get('content') is not None else 'x'
            item[ITEM_TYPE] = CONTENT_TYPE.TEXT if anpa_format == 'x' else CONTENT_TYPE.PREFORMATTED


class NITFFeedParser(XMLFeedParser):
    """
    Feed Parser which can parse if the feed is in NITF format.
    """

    NAME = 'nitf'

    def can_parse(self, xml):
        return xml.tag == 'nitf'

    def parse(self, xml, provider=None):
        item = {}
        try:
            docdata = xml.find('head/docdata')
            # set the default type.
            item[ITEM_TYPE] = CONTENT_TYPE.TEXT
            item['guid'] = item['uri'] = docdata.find('doc-id').get('id-string')
            if docdata.find('urgency') is not None:
                item['urgency'] = int(docdata.find('urgency').get('ed-urg', '5'))
            item['pubstatus'] = (docdata.attrib.get('management-status', 'usable')).lower()
            item['firstcreated'] = get_norm_datetime(docdata.find('date.issue'))
            item['versioncreated'] = get_norm_datetime(docdata.find('date.issue'))

            if docdata.find('date.expire') is not None:
                item['expiry'] = get_norm_datetime(docdata.find('date.expire'))
            item['subject'] = get_subjects(xml)
            item['body_html'] = get_content(xml)
            item['place'] = get_places(docdata)
            item['keywords'] = get_keywords(docdata)

            if docdata.find('ed-msg') is not None:
                item['ednote'] = docdata.find('ed-msg').attrib.get('info')

            if xml.find('body/body.head/hedline/hl1') is not None:
                item['headline'] = xml.find('body/body.head/hedline/hl1').text
            else:
                if xml.find('head/title') is not None:
                    item['headline'] = xml.find('head/title').text

            elem = xml.find('body/body.head/abstract')
            item['abstract'] = elem.text if elem is not None else ''

            elem = xml.find('body/body.head/dateline/location/city')
            if elem is not None:
                self.set_dateline(item, city=elem.text)

            item['byline'] = get_byline(xml)

            parse_meta(xml, item)
            item.setdefault('word_count', get_word_count(item['body_html']))
            return item
        except Exception as ex:
            raise ParserError.nitfParserError(ex, provider)


register_feed_parser(NITFFeedParser.NAME, NITFFeedParser())



import os
import json
import superdesk
import pymongo
import logging

from superdesk import get_resource_service
from flask import current_app as app
from re import findall

logger = logging.getLogger(__name__)

"""
App initialization information, maps resource name to the file containing the data
and the index to be created for the resource and the boolean flag to update the
resource or not.
__entities__ = {
    "resource_name": ("file_name", "index_params", "do_patch")
}
"file_name" (str): name of the file containing seed data
"index_params list: List of key (field or List of tuple as required by pymongo create_index function.
http://api.mongodb.org/python/current/api/pymongo/collection.html
For example:
[[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], "username"] will create two indexes
- composite index of "first_name", "last_name" field.
- index on username field.
Alternatively index param can be specified as
[[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], [("username", pymongo.ASCENDING)]]
Options can be sent to index creation and in this case the last element in the list is the options
dictionary:
[[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], {'sparse': True}]]
"""
__entities__ = {
    'roles': ('roles.json', ['name'], False),
    'users': ('users.json', [[('first_name', pymongo.ASCENDING),
                             ('last_name', pymongo.DESCENDING)],
                             'username'], False),
    'desks': ('desks.json', ['incoming_stage'], False),
    'stages': ('stages.json', ['desk'], False),
    'groups': ('groups.json', '', False),
    'vocabularies': ('vocabularies.json', '', True),
    'validators': ('validators.json', '', True),
    'content_templates': ('content_templates.json', ['template_name'], False),
    'ingest_providers': ('ingest_providers.json', '', True),
    'published': (None, [[('expiry', pymongo.ASCENDING),
                          ('_created', pymongo.ASCENDING),
                          ('state', pymongo.ASCENDING)],
                         [('item_id', pymongo.ASCENDING),
                          ('state', pymongo.ASCENDING)],
                         [('publish_schedule', pymongo.ASCENDING),
                          ('state', pymongo.ASCENDING)]], False),
    'activity': (None, [[('_created', pymongo.DESCENDING),
                         {'expireAfterSeconds': 86400}],
                        [('item', pymongo.ASCENDING),
                         ('read', pymongo.ASCENDING),
                         ('user', pymongo.ASCENDING)],
                        [('resource', pymongo.ASCENDING),
                         ('data.provider_id', pymongo.ASCENDING)]], False),
    'archive': (None, [[('_updated', pymongo.ASCENDING)],
                       [('expiry', pymongo.ASCENDING),
                        ('state', pymongo.ASCENDING)],
                       [('type', pymongo.ASCENDING)],
                       [('groups.refs.residRef', pymongo.ASCENDING), {'sparse': True}],
                       [('publish_schedule', pymongo.ASCENDING),
                        ('state', pymongo.ASCENDING)],
                       [('unique_name', pymongo.ASCENDING)]], False),
    'archive_versions': (None, [[('_id_document', pymongo.ASCENDING),
                                 ('_current_version', pymongo.ASCENDING)]], False),
    'ingest': (None, [[('expiry', pymongo.ASCENDING),
                       ('ingest_provider', pymongo.ASCENDING)],
                      [('guid', pymongo.ASCENDING)]], False),
    'publish_queue': (None, [[('_created', pymongo.DESCENDING),
                              ('state', pymongo.ASCENDING),
                              ('destination.delivery_type', pymongo.ASCENDING)],
                             [('item_id', pymongo.ASCENDING),
                              ('item_version', pymongo.ASCENDING)],
                             [('state', pymongo.ASCENDING),
                              ('destination.delivery_type', pymongo.ASCENDING)],
                             [('subscriber_id', pymongo.ASCENDING)],
                             [('_updated', pymongo.DESCENDING)]], False),
    'archived': (None, [[('archived_id', pymongo.ASCENDING), {'unique': True}]], False),
    'legal_archive_versions': (None, [[('_id_document', pymongo.ASCENDING),
                                       ('_current_version', pymongo.ASCENDING)]], False),
    'legal_publish_queue': (None, [[('_updated', pymongo.DESCENDING)]], False)
}


def get_filepath(filename):
    basedir = app.config.get(
        'INIT_DATA_PATH',
        os.path.join(os.path.abspath(os.path.dirname(__file__)), 'data_initialization')
    )

    return os.path.join(basedir, filename)


class AppInitializeWithDataCommand(superdesk.Command):
    """
    Initialize application with predefined data for various entities.
    Entities supported: [roles, users, desks, stages, groups, vocabularies, validators, content_templates].
    If no --entity-name parameter is supplied, all the entities are inserted.
    The entities [vocabularies, validators] will be updated with the predefined data if it already exists,
    no action will be taken for the other entities.
    """

    option_list = [
        superdesk.Option('--entity-name', '-n', dest='entity_name', default='')
    ]

    def run(self, entity_name=None, index_only='false'):
        logger.info('Starting data import')
        if entity_name:
            (file_name, index_params, do_patch) = __entities__[entity_name]
            self.import_file(entity_name, file_name, index_params, do_patch)
            return 0

        for name, (file_name, index_params, do_patch) in __entities__.items():
            try:
                self.import_file(name, file_name, index_params, do_patch)
            except Exception as ex:
                logger.info('Exception loading entity {} from {}'.format(name, file_name))
                logger.exception(ex)

        logger.info('Data import finished')
        return 0

    def import_file(self, entity_name, file_name, index_params, do_patch=False):
        """
        imports seed data based on the entity_name (resource name) from the file_name specified.
        index_params use to create index for that entity/resource
        :param str entity_name: name of the resource
        :param str file_name: file name that contains seed data
        :param list index_params: list of indexes that is created on that entity.
        For example:
        [[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], "username"] will create two indexes
        - composite index of "first_name", "last_name" field.
        - index on username field.
        Alternatively index param can be specified as
        [[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], [("username", pymongo.ASCENDING)]]
        Refer to pymongo create_index documentation for more information.
        http://api.mongodb.org/python/current/api/pymongo/collection.html
        :param bool do_patch: if True then patch the document else don't patch.
        """
        print('Config: ', app.config['APP_ABSPATH'])
        if file_name:
            file_path = get_filepath(file_name)
            print('Got file path: ', file_path)
            with open(file_path, 'rt') as app_prepopulation:
                service = get_resource_service(entity_name)
                json_data = json.loads(app_prepopulation.read())
                data = [fillEnvironmentVariables(item) for item in json_data]
                data = [app.data.mongo._mongotize(item, service.datasource) for item in data if item]
                existing_data = []
                existing = service.get_from_mongo(None, {})
                update_data = True
                if not do_patch and existing.count() > 0:
                    logger.info('Data already exists for {} none will be loaded'.format(entity_name))
                    update_data = False
                elif do_patch and existing.count() > 0:
                    logger.info('Data already exists for {} it will be updated'.format(entity_name))

                if update_data:
                    if do_patch:
                        for item in existing:
                            for loaded_item in data:
                                if '_id' in loaded_item and loaded_item['_id'] == item['_id']:
                                    existing_data.append(loaded_item)
                                    data.remove(loaded_item)

                    if data:
                        service.post(data)

                    if existing_data and do_patch:
                        for item in existing_data:
                            service.patch(item['_id'], item)

                logger.info('File {} imported successfully.'.format(file_name))

        if index_params:
            for index in index_params:
                crt_index = list(index) if isinstance(index, list) else index
                options = crt_index.pop() if isinstance(crt_index[-1], dict) and isinstance(index, list) else {}
                collection = app.data.mongo.pymongo(resource=entity_name).db[entity_name]
                index_name = collection.create_index(crt_index, cache_for=300, **options)
                logger.info('Index: {} for collection {} created successfully.'.format(index_name, entity_name))


def fillEnvironmentVariables(item):
    variables = {}
    text = json.dumps(item)

    for variable in findall('#ENV_([^#"]+)#', text):
        value = os.environ.get(variable, None)
        if not value:
            return None
        else:
            variables[variable] = value

    for name in variables:
        text = text.replace('#ENV_%s#' % name, variables[name])

    return json.loads(text)


superdesk.command('app:initialize_data', AppInitializeWithDataCommand())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from superdesk.notification import push_notification

import superdesk
import logging
from eve.utils import ParsedRequest, date_to_str, config

from apps.packages import PackageService
from superdesk.celery_task_utils import is_task_running, mark_task_as_not_running, get_lock_id
from superdesk.utc import utcnow
from .archive import SOURCE as ARCHIVE
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE, ITEM_TYPE, CONTENT_TYPE
from superdesk.metadata.packages import PACKAGE_TYPE, TAKES_PACKAGE
from superdesk.lock import lock, unlock
from superdesk import get_resource_service

logger = logging.getLogger(__name__)

UPDATE_OVERDUE_SCHEDULED_DEFAULT = {'minutes': 10}


class UpdateOverdueScheduledContent(superdesk.Command):
    """
    Updates the overdue scheduled stories
    """

    def run(self):
        self.update_overdue_scheduled()

    def update_overdue_scheduled(self):
        """
        Updates the overdue scheduled content on archive collection.
        """

        logger.info('Updating overdue scheduled content')

        if is_task_running("archive", "update_overdue_scheduled", UPDATE_OVERDUE_SCHEDULED_DEFAULT):
            return

        try:
            now = date_to_str(utcnow())
            items = get_overdue_scheduled_items(now, ARCHIVE)
            item_update = {ITEM_STATE: CONTENT_STATE.PUBLISHED}

            for item in items:
                logger.info('updating overdue scheduled article with id {} and headline {} -- expired on: {} now: {}'.
                            format(item[config.ID_FIELD], item['headline'], item['publish_schedule'], now))

                superdesk.get_resource_service(ARCHIVE).patch(item[config.ID_FIELD], item_update)
        finally:
            mark_task_as_not_running("archive", "update_overdue_scheduled")


class RemoveExpiredContent(superdesk.Command):

    def run(self):
        now = utcnow()
        expiry_time_log_msg = 'Expiry Time: {}.'.format(now)
        logger.info('{} Starting to remove expired content at.'.format(expiry_time_log_msg))
        lock_name = get_lock_id('archive', 'remove_expired')
        if not lock(lock_name, '', expire=600):
            logger.info('{} Remove expired content task is already running.'.format(expiry_time_log_msg))
            return
        try:
            logger.info('{} Removing expired content for expiry.'.format(expiry_time_log_msg))
            self._remove_expired_items(now, expiry_time_log_msg)
        finally:
            unlock(lock_name, '')

        push_notification('content:expired')
        logger.info('{} Completed remove expired content.'.format(expiry_time_log_msg))

    def _remove_expired_items(self, expiry_datetime, log_msg):
        """
        Remove the expired items.
        :param datetime expiry_datetime: expiry datetime
        :param str log_msg: log message to be prefixed
        """
        logger.info('{} Starting to remove published expired items.'.format(log_msg))
        archive_service = get_resource_service(ARCHIVE)
        published_service = get_resource_service('published')

        expired_items = list(archive_service.get_expired_items(expiry_datetime))
        if len(expired_items) == 0:
            logger.info('{} No items found to expire.'.format(log_msg))
            return

        # Step 1: Get the killed Items
        spiked_killed_items = [item for item in expired_items
                               if item.get(ITEM_STATE) in {CONTENT_STATE.KILLED, CONTENT_STATE.SPIKED}]

        items_to_remove = set()
        items_to_be_archived = set()

        # Step 2: Get the not killed and spiked items
        not_killed_items = [item for item in expired_items
                            if item.get(ITEM_STATE) not in {CONTENT_STATE.KILLED, CONTENT_STATE.SPIKED}]

        log_msg_format = "{{'_id': {_id}, 'unique_name': {unique_name}, 'version': {_current_version}, " \
                         "'expired_on': {expiry}}}."

        # Step 3: Processing items to expire
        for item in not_killed_items:
            item_id = item.get(config.ID_FIELD)
            expiry_msg = log_msg_format.format(**item)
            logger.info('{} Processing expired item. {}'.format(log_msg, expiry_msg))

            processed_items = set()
            if item_id not in items_to_be_archived and self._can_remove_item(item, processed_items):
                # item can be archived and removed from the database
                logger.info('{} Removing item. {}'.format(log_msg, expiry_msg))
                logger.info('{} Items to be removed. {}'.format(log_msg, processed_items))
                items_to_be_archived = items_to_be_archived | processed_items

        # move to archived collection
        logger.info('{} Archiving items.'.format(log_msg))
        for item in items_to_be_archived:
            self._move_to_archived(item, log_msg)

        for item in spiked_killed_items:
            # delete from the published collection and queue
            msg = log_msg_format.format(**item)
            try:
                if item.get(ITEM_STATE) == CONTENT_STATE.KILLED:
                    published_service.delete_by_article_id(item.get(config.ID_FIELD))
                    logger.info('{} Deleting killed item from published. {}'.format(log_msg, msg))

                items_to_remove.add(item.get(config.ID_FIELD))
            except:
                logger.exception('{} Failed to delete killed item from published. {}'.format(log_msg, msg))

        if items_to_remove:
            logger.info('{} Deleting articles.: {}'.format(log_msg, items_to_remove))
            archive_service.delete_by_article_ids(list(items_to_remove))

        logger.info('{} Deleting killed and spiked items from archive.'.format(log_msg))

    def _can_remove_item(self, item, processed_item=None):
        """
        Recursively checks if the item can be removed.
        :param dict item: item to be remove
        :param set processed_item: processed items
        :return: True if item can be removed, False otherwise.
        """

        if processed_item is None:
            processed_item = set()

        item_refs = []
        package_service = PackageService()
        archive_service = get_resource_service(ARCHIVE)

        if item.get(ITEM_TYPE) == CONTENT_TYPE.COMPOSITE:
            # Get the item references for is package
            item_refs = package_service.get_residrefs(item)

        if item.get(PACKAGE_TYPE) == TAKES_PACKAGE or \
           item.get(ITEM_TYPE) in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            broadcast_items = get_resource_service('archive_broadcast').get_broadcast_items_from_master_story(item)
            # If master story expires then check if broadcast item is included in a package.
            # If included in a package then check the package expiry.
            item_refs.extend([broadcast_item.get(config.ID_FIELD) for broadcast_item in broadcast_items])

        # get item reference where this referred
        item_refs.extend(package_service.get_linked_in_package_ids(item))

        # check item refs in the ids to remove set
        is_expired = item.get('expiry') < utcnow()

        if is_expired:
            # now check recursively for all references
            if item.get(config.ID_FIELD) in processed_item:
                return is_expired

            processed_item.add(item.get(config.ID_FIELD))
            if item_refs:
                archive_items = archive_service.get_from_mongo(req=None, lookup={'_id': {'$in': item_refs}})
                for archive_item in archive_items:
                    is_expired = self._can_remove_item(archive_item, processed_item)
                    if not is_expired:
                        break

        return is_expired

    def _move_to_archived(self, _id, log_msg):
        """
        Moves all the published version of an article to archived.
        Deletes all published version of an article in the published collection
        :param str _id: id of the document to be moved
        :param str log_msg: log message to be prefixed
        """
        published_service = get_resource_service('published')
        archived_service = get_resource_service('archived')
        archive_service = get_resource_service('archive')

        published_items = list(published_service.get_from_mongo(req=None, lookup={'item_id': _id}))

        try:
            if published_items:
                archived_service.post(published_items)
                logger.info('{} Archived published item'.format(log_msg))
                published_service.delete_by_article_id(_id)
                logger.info('{} Deleted published item.'.format(log_msg))

            archive_service.delete_by_article_ids([_id])
            logger.info('{} Delete archive item.'.format(log_msg))
        except:
            failed_items = [item.get(config.ID_FIELD) for item in published_items]
            logger.exception('{} Failed to move to archived. {}'.format(log_msg, failed_items))


def get_overdue_scheduled_items(expired_date_time, resource, limit=100):
    """
    Fetches the overdue scheduled articles from given collection. Overdue Conditions:
        1.  it should be in 'scheduled' state
        2.  publish_schedule is less than or equal to expired_date_time

    :param expired_date_time: DateTime that scheduled tate will be checked against
    :param resource: Name of the resource to check the data from
    :param limit: Number of return items
    :return: overdue scheduled articles from published collection
    """

    logger.info('Get overdue scheduled content from {}'.format(resource))
    query = {'$and': [
        {'publish_schedule': {'$lte': expired_date_time}},
        {ITEM_STATE: CONTENT_STATE.SCHEDULED}
    ]}

    req = ParsedRequest()
    req.sort = '_modified'
    req.max_results = limit

    return superdesk.get_resource_service(resource).get_from_mongo(req=req, lookup=query)

superdesk.command('archive:remove_overdue_scheduled', UpdateOverdueScheduledContent())
superdesk.command('archive:remove_expired', RemoveExpiredContent())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


class FieldMapper():
    """Base mapper class can be used in all formatters"""

    def map(self, article, category, **kwargs):
        """Formats the article and returns the transformed string"""
        raise NotImplementedError()




import superdesk


CONTENT_TYPE_PRIVILEGE = 'content_type'


class ContentTypesResource(superdesk.Resource):
    schema = {
        '_id': {
            'type': 'string',
            'unique': True,
        },
        'label': {
            'type': 'string',
        },
        'schema': {
            'type': 'dict',
        },
        'priority': {
            'type': 'integer',
            'default': 0,
        },
        'enabled': {
            'type': 'boolean',
            'default': True,
        },
    }

    item_url = 'regex("[\w,.:-]+")'

    privileges = {'POST': CONTENT_TYPE_PRIVILEGE,
                  'PATCH': CONTENT_TYPE_PRIVILEGE,
                  'DELETE': CONTENT_TYPE_PRIVILEGE}

    datasource = {
        'default_sort': [('priority', -1)],
    }


class ContentTypesService(superdesk.Service):
    pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.emails import send_email
from flask import current_app as app
from superdesk.publish import register_transmitter
from superdesk.publish.publish_service import PublishService
from superdesk.errors import PublishEmailError

errors = [PublishEmailError.emailError().get_error_description()]


class EmailPublishService(PublishService):
    """Email Publish Service."""

    def _transmit(self, queue_item, subscriber):
        config = queue_item.get('destination', {}).get('config', {})

        try:
            if not config.get('recipients'):
                raise PublishEmailError.recipientNotFoundError(LookupError('recipient field not found!'))

            admins = app.config['ADMINS']
            recipients = config.get('recipients').rstrip(';').split(';')
            subject = "Story: {}".format(queue_item['item_id'])
            text_body = queue_item['formatted_item']

            # sending email synchronously
            send_email(subject=subject,
                       sender=admins[0],
                       recipients=recipients,
                       text_body=text_body,
                       html_body=None)

        except PublishEmailError:
            raise
        except Exception as ex:
            raise PublishEmailError.emailError(ex, queue_item.get('destination'))

register_transmitter('email', EmailPublishService(), errors)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.notification import push_notification
from superdesk.services import BaseService
import superdesk


def init_app(app):
    endpoint_name = 'coverages'
    service = CoverageService(endpoint_name, backend=superdesk.get_backend())
    CoverageResource(endpoint_name, app=app, service=service)


class CoverageResource(Resource):
    schema = {
        'headline': {'type': 'string'},
        'coverage_type': {
            'type': 'string',
            'allowed': ['story', 'photo', 'video', 'graphics', 'live-blogging'],
            'default': 'story',
        },
        'ed_note': {'type': 'string'},
        'scheduled': {'type': 'datetime'},
        'delivery': {'type': 'string'},
        'assigned_user': Resource.rel('users', True),
        'assigned_desk': Resource.rel('desks', True),
        'planning_item': Resource.rel('planning', True, type='string'),
    }

    datasource = {'default_sort': [('_created', -1)]}
    privileges = {'POST': 'planning', 'PATCH': 'planning', 'DELETE': 'planning'}


class CoverageService(BaseService):

    def on_created(self, docs):
        push_notification('coverages', created=1)

    def on_updated(self, updates, original):
        push_notification('coverages', updated=1)

    def on_deleted(self, doc):
        push_notification('coverages', deleted=1)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime
import traceback

import requests
from flask import current_app as app

from superdesk.errors import IngestApiError
from superdesk.etree import etree, ParseError
from superdesk.io import register_feeding_service
from superdesk.io.feeding_services.http_service import HTTPFeedingService
from superdesk.logging import logger
from superdesk.utc import utcnow
from urllib.parse import urlparse, urlunparse


class ReutersHTTPFeedingService(HTTPFeedingService):
    """
    Feeding Service class which can read article(s) using HTTP provided by Reuters.
    """

    NAME = 'reuters_http'

    ERRORS = [IngestApiError.apiTimeoutError().get_error_description(),
              IngestApiError.apiRedirectError().get_error_description(),
              IngestApiError.apiRequestError().get_error_description(),
              IngestApiError.apiUnicodeError().get_error_description(),
              IngestApiError.apiParseError().get_error_description(),
              IngestApiError.apiGeneralError().get_error_description()]

    DATE_FORMAT = '%Y.%m.%d.%H.%M'

    def _update(self, provider):
        updated = utcnow()

        last_updated = provider.get('last_updated')
        ttl_minutes = app.config['INGEST_EXPIRY_MINUTES']
        if not last_updated or last_updated < updated - datetime.timedelta(minutes=ttl_minutes):
            last_updated = updated - datetime.timedelta(minutes=ttl_minutes)

        self.provider = provider
        provider_config = provider.get('config')
        if not provider_config:
            provider_config = {}
            provider['config'] = provider_config

        if 'url' not in provider_config:
            provider_config['url'] = 'http://rmb.reuters.com/rmd/rest/xml'

        if 'auth_url' not in provider_config:
            provider_config['auth_url'] = 'https://commerce.reuters.com/rmd/rest/xml/login'

        self.URL = provider_config.get('url')

        for channel in self._get_channels():
            for guid in self._get_article_ids(channel, last_updated, updated):
                items = self.fetch_ingest(guid)
                if items:
                    yield items

    def _get_channels(self):
        """Get subscribed channels."""
        channels = []
        tree = self._get_tree('channels')
        for channel in tree.findall('channelInformation'):
            channels.append(channel.find('alias').text)

        return channels

    def _get_tree(self, endpoint, payload=None):
        """
        Get xml response for given API endpoint and payload.
        :param: endpoint
        :type endpoint: str
        :param: payload
        :type payload: str
        """

        if payload is None:
            payload = {}

        payload['token'] = self._get_auth_token(self.provider, update=True)
        url = self._get_absolute_url(endpoint)

        try:
            response = requests.get(url, params=payload, timeout=15)
        except requests.exceptions.Timeout as ex:
            # Maybe set up for a retry, or continue in a retry loop
            raise IngestApiError.apiTimeoutError(ex, self.provider)
        except requests.exceptions.TooManyRedirects as ex:
            # Tell the user their URL was bad and try a different one
            raise IngestApiError.apiRedirectError(ex, self.provider)
        except requests.exceptions.RequestException as ex:
            # catastrophic error. bail.
            raise IngestApiError.apiRequestError(ex, self.provider)
        except Exception as error:
            traceback.print_exc()
            raise IngestApiError.apiGeneralError(error, self.provider)

        if response.status_code == 404:
            raise LookupError('Not found %s' % payload)

        try:
            return etree.fromstring(response.content)  # workaround for http mock lib
        except UnicodeEncodeError as error:
            traceback.print_exc()
            raise IngestApiError.apiUnicodeError(error, self.provider)
        except ParseError as error:
            traceback.print_exc()
            raise IngestApiError.apiParseError(error, self.provider)
        except Exception as error:
            traceback.print_exc()
            raise IngestApiError.apiGeneralError(error, self.provider)

    def _get_absolute_url(self, endpoint):
        """
        Get absolute URL for given endpoint.

        :param: endpoint
        :type endpoint: str
        """
        return '/'.join([self.URL, endpoint])

    def _get_article_ids(self, channel, last_updated, updated):
        """
        Get article ids which should be upserted.
        """

        ids = set()
        payload = {'channel': channel, 'fieldsRef': 'id',
                   'dateRange': "%s-%s" % (self._format_date(last_updated), self._format_date(updated))}

        logger.info('Reuters requesting Date Range |{}| for channel {}'.format(payload['dateRange'], channel))
        tree = self._get_tree('items', payload)
        for result in tree.findall('result'):
            ids.add(result.find('guid').text)

        return ids

    def _format_date(self, date):
        return date.strftime(self.DATE_FORMAT)

    def fetch_ingest(self, guid):
        items = self._parse_items(guid)
        result_items = []
        while items:
            item = items.pop()
            self.add_timestamps(item)
            try:
                items.extend(self._fetch_items_in_package(item))
                result_items.append(item)
            except LookupError as err:
                self.log_item_error(err, item, self.provider)
                return []

        return result_items

    def _parse_items(self, guid):
        """
        Parse item message and return given items.
        """

        payload = {'id': guid}
        tree = self._get_tree('item', payload)

        parser = self.get_feed_parser(self.provider, tree)
        items = parser.parse(tree, self.provider)

        return items

    def _fetch_items_in_package(self, item):
        """
        Fetch remote assets for given item.
        """
        items = []
        for group in item.get('groups', []):
            for ref in group.get('refs', []):
                if 'residRef' in ref:
                    items.extend(self._parse_items(ref.get('residRef')))

        return items

    def prepare_href(self, href):
        (scheme, netloc, path, params, query, fragment) = urlparse(href)
        new_href = urlunparse((scheme, netloc, path, '', '', ''))
        return '%s?auth_token=%s' % (new_href, self._get_auth_token(self.provider, update=True))


register_feeding_service(ReutersHTTPFeedingService.NAME, ReutersHTTPFeedingService(), ReutersHTTPFeedingService.ERRORS)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import unittest
from macros.dollar_AUD_character_replace import find_and_replace


class DollarAUDReplaceTestCase(unittest.TestCase):

    def setUp(self):
        self.body = ("Cost of train tickets increased by $1 for all destinations")

    def test_replace_body(self):
        item = {'body_html': self.body}
        item = find_and_replace(item)
        self.assertEqual(item['body_html'], 'Cost of train tickets increased by AUD 1 for all destinations')

    def test_replace_abstract(self):
        item = {'abstract': self.body}
        item = find_and_replace(item)
        self.assertEqual(item['abstract'], 'Cost of train tickets increased by AUD 1 for all destinations')

    def test_replace_headline(self):
        item = {'headline': self.body}
        item = find_and_replace(item)
        self.assertEqual(item['headline'], 'Cost of train tickets increased by AUD 1 for all destinations')

    def test_replace_slugline(self):
        item = {'slugline': self.body}
        item = find_and_replace(item)
        self.assertEqual(item['slugline'], 'Cost of train tickets increased by AUD 1 for all destinations')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import current_app as app, json, g
from eve_elastic.elastic import set_filters

import superdesk
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE
from superdesk.metadata.utils import aggregations
from apps.archive.archive import SOURCE as ARCHIVE
from superdesk.resource import build_custom_hateoas


class SearchService(superdesk.Service):
    """
    Federated search service.

    It can search against different collections like Ingest, Production, Archived etc.. at the same time.
    """

    def __init__(self, datasource, backend):
        super().__init__(datasource=datasource, backend=backend)
        self._default_repos = ['ingest', 'archive', 'published', 'archived']

        self._private_filters = dict(ingest={'and': [{'term': {'_type': 'ingest'}}]},
                                     archive={'and': [{'exists': {'field': 'task.desk'}},
                                                      {'terms': {
                                                          ITEM_STATE: [CONTENT_STATE.FETCHED, CONTENT_STATE.DRAFT,
                                                                       CONTENT_STATE.ROUTED, CONTENT_STATE.PROGRESS,
                                                                       CONTENT_STATE.SUBMITTED]}}]},
                                     published={'and': [{'term': {'_type': 'published'}},
                                                        {'terms': {ITEM_STATE: [CONTENT_STATE.SCHEDULED,
                                                                                CONTENT_STATE.PUBLISHED,
                                                                                CONTENT_STATE.KILLED,
                                                                                CONTENT_STATE.CORRECTED]}}]},
                                     archived={'and': [{'term': {'_type': 'archived'}}]})

    def _get_query(self, req):
        """Get elastic query."""
        args = getattr(req, 'args', {})
        query = json.loads(args.get('source')) if args.get('source') else {'query': {'filtered': {}}}
        query['aggs'] = aggregations
        return query

    def _get_types(self, req):
        """Get document types for the given query."""
        args = getattr(req, 'args', {})
        repos = args.get('repo')

        if repos is None:
            return self._default_repos.copy()
        else:
            repos = repos.split(',')
            return [repo for repo in repos if repo in self._default_repos]

    def _get_filters(self, repos):
        """
        Gets filters for the passed repos.
        """
        filters = []

        for repo in repos:
            filters.append(self._private_filters[repo])

        return [{'or': filters}]

    def get(self, req, lookup):
        """
        Runs elastic search on multiple doc types.
        """

        elastic = app.data.elastic
        query = self._get_query(req)
        types = self._get_types(req)
        filters = self._get_filters(types)

        query['aggs'] = aggregations

        stages = superdesk.get_resource_service('users').get_invisible_stages_ids(g.get('user', {}).get('_id'))
        if stages:
            filters.append({'and': [{'not': {'terms': {'task.stage': stages}}}]})

        # if the system has a setting value for the maximum search depth then apply the filter
        if not app.settings['MAX_SEARCH_DEPTH'] == -1:
            filters.append(dict(limit={'value': app.settings['MAX_SEARCH_DEPTH']}))

        set_filters(query, filters)

        hits = elastic.es.search(body=query, index=elastic.index, doc_type=types)
        docs = elastic._parse_hits(hits, 'ingest')  # any resource here will do

        for resource in types:
            response = {app.config['ITEMS']: [doc for doc in docs if doc['_type'] == resource]}
            getattr(app, 'on_fetched_resource')(resource, response)
            getattr(app, 'on_fetched_resource_%s' % resource)(response)

        return docs

    def on_fetched(self, doc):
        """
        Overriding to add HATEOS for each individual item in the response.

        :param doc: response doc
        :type doc: dict
        """

        docs = doc[app.config['ITEMS']]
        for item in docs:
            build_custom_hateoas({'self': {'title': item['_type'], 'href': '/{}/{{_id}}'.format(item['_type'])}}, item)


class SearchResource(superdesk.Resource):
    resource_methods = ['GET']
    item_methods = []


def init_app(app):
    search_service = SearchService(ARCHIVE, backend=superdesk.get_backend())
    SearchResource('search', app=app, service=search_service)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from test_factory import SuperdeskTestCase
from apps.publish import init_app
from apps.publish.formatters.aap_sms_formatter import AAPSMSFormatter


class AapSMSFormatterTest(SuperdeskTestCase):
    subscribers = [{"_id": "1", "name": "Test", "can_send_takes_packages": False, "media_type": "media",
                    "is_active": True, "sequence_num_settings": {"max": 10, "min": 1},
                    "destinations": [{"name": "AAP SMS", "delivery_type": "ODBC", "format": "AAP SMS",
                                      "config": {"connection_string": "DRIVER=BLAH", "stored_procedure": "Insert"}
                                      }]
                    }]

    article = {
        'priority': 1,
        'anpa_category': [{'qcode': 'a'}],
        'headline': 'This is a test headline',
        'type': 'text',
        'body_html': 'The story body',
        'body_footer': 'call helpline 999 if you are planning to quit smoking'
    }

    def setUp(self):
        super().setUp()
        self.app.data.insert('subscribers', self.subscribers)
        init_app(self.app)

    def TestSMSFormatter(self):
        subscriber = self.app.data.find('subscribers', None, None)[0]

        f = AAPSMSFormatter()
        seq, item = f.format(self.article, subscriber)[0]

        self.assertGreater(int(seq), 0)
        self.assertDictEqual(item, {'Category': 'a', 'Priority': 'f', 'Sequence': item['Sequence'], 'ident': '0',
                                    'Headline': 'This is a test headline',
                                    'StoryText':
                                        'The story bodycall helpline 999 if you are planning to quit smoking'})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.publish.formatters.field_mappers import FieldMapper


class LocatorMapper(FieldMapper):

    iptc_locators = {
        '01020000': 'ARTS',
        '01010000': 'BOOKS',
        '08003002': 'CELEB',
        '01007000': 'FASH',
        '01005000': 'FILM',
        '07000000': 'HLTH',
        '10014000': 'MOTOR',
        '01011000': 'MUSIC',
        '01014000': 'RADIO',
        '13000000': 'SCI',
        '13010000': 'TECH',
        '10006000': 'TRAV',
        '01016000': 'TV'
    }

    iptc_sports_locators = {
        '15084000': 'AFL',
        '15005000': 'ATHS',
        '15006000': 'BAD',
        '15007000': 'BASE',
        '15008000': 'BASK',
        '15013000': 'BOWLS',
        '15014000': 'BOX',
        '15067001': 'BVO',
        '15017000': 'CRIK',
        '15019000': 'CYCLE',
        '15100000': 'DAR',
        '15021000': 'DIVE',
        '15022000': 'EQN',
        '15023000': 'FENCE',
        '15073005': 'GAMES',
        '15027000': 'GOLF',
        '15028000': 'GYM',
        '15029000': 'HBL',
        '15024000': 'HOCK',
        '15031000': 'ICE',
        '15066000': 'TRI',  # and IRON
        '15033000': 'JUDO',
        '15015000': 'KAYAK',  # and CANOE
        '15039000': 'MOTOR',
        '15042000': 'NET',
        '15003001': 'NFL',
        '15073001': 'OLY',
        '15073002': 'OLY',
        '15073047': 'PARA',
        '15038000': 'PENT',
        '15045000': 'POLO',
        '15030000': 'RACE',
        '15048000': 'RL',
        '15047000': 'ROW',
        '15049000': 'RU',
        '15050000': 'SAIL',
        '15051000': 'SHOOT',
        '15002000': 'SKI',
        '15010000': 'SNOOK',
        '15000000': 'SPO',
        '15054000': 'SOC',
        '15055000': 'SOFT',
        '15059000': 'SQSH',
        '15062025': 'SSWIM',
        '15062026': 'SSWIM',
        '15061000': 'SURF',
        '15062000': 'SWIM',
        '15064000': 'TAE',
        '15065000': 'TEN',
        '15063000': 'TTEN',
        '15067000': 'VOL',
        '15070000': 'WGHT',
        '15068000': 'WPOL',
        '15072000': 'WRES'
    }

    def map(self, article, category, **kwargs):
        """
        Based on the category and subject code it returns the locator
        :param dict article: original article
        :param str category: category of the article
        :param dict kwargs: keyword args
        :return: if found then the locator as string else None
        """
        if category == 'S':
            mapped_value = self._map_locator_code(article, category, self.iptc_sports_locators)
        else:
            mapped_value = self._map_locator_code(article, category, self.iptc_locators)

        # If no mapping is found then use the first place value
        if not mapped_value:
            for place in (article.get('place') or []):
                return place.get('qcode')

        return mapped_value

    def _map_locator_code(self, article, category, locators):
        """
        Based on the category and subject code it returns the locator
        :param dict article: original article
        :param str category: category of the article
        :param dict locators: subject code locator mapping dictionary
        :return: if found then the locator as string else None
        """
        subjects = article.get('subject') or []
        for subject in subjects:
            qcode = subject.get('qcode', '')
            feature = locators.get(qcode)
            if feature:
                if qcode == '10006000':
                    return '{}{}'.format(feature, 'I' if category == 'I' else 'D')
                else:
                    return feature
        return None



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
import unittest

from superdesk.io.feed_parsers.zczc import ZCZCFeedParser


class ZCZCTestCase(unittest.TestCase):
    provider = {'name': 'test provder', 'provider': {}}

    def test_default_format(self):
        filename = 'Standings__2014_14_635535729050675896.tst'
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', filename))
        self.provider['source'] = 'SOMETHING'
        self.items = ZCZCFeedParser().parse(fixture, self.provider)
        self.assertEqual(self.items.get('headline'), 'MOTOR:  Collated results/standings after Sydney NRMA 500')
        self.assertEqual(self.items.get('anpa_category')[0]['qcode'], 'T')
        self.assertEqual(self.items.get('subject')[0]['qcode'], '15039001')
        self.assertIn('versioncreated', self.items)

    def test_medianet_format(self):
        filename = 'ED_841066_2_1.tst'
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', filename))
        self.provider['source'] = 'MNET'
        self.items = ZCZCFeedParser().parse(fixture, self.provider)
        self.assertEqual(self.items.get('headline'), 'Australian Financial Security Authority')

    def test_pagemasters_format(self):
        filename = 'Darwin GR - Greys - Sun 11 Oct, 2015.tst'
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', filename))
        self.provider['source'] = 'PMF'
        self.items = ZCZCFeedParser().parse(fixture, self.provider)
        self.assertEqual(self.items.get('headline'), 'Darwin Greyhound Fields Sunday')
        self.assertEqual(self.items.get('slugline'), 'Darwin Grey')
        self.assertEqual(self.items.get('anpa_category')[0]['qcode'], 'r')
        self.assertEqual(self.items.get('subject')[0]['qcode'], '15082002')

    def test_racing_format(self):
        filename = 'viflda004_7257.tst'
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', filename))
        self.provider['source'] = 'BRA'
        self.items = ZCZCFeedParser().parse(fixture, self.provider)
        self.assertEqual(self.items.get('headline'), ' Racing.Com Park FIELDS Thursday')
        self.assertEqual(self.items.get('anpa_category')[0]['qcode'], 'r')
        self.assertEqual(self.items.get('subject')[0]['qcode'], '15030001')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json

from eve.utils import config
import requests

from superdesk import app
from superdesk.errors import PublishHTTPPushError
from superdesk.publish import register_transmitter
from superdesk.publish.publish_service import PublishService


errors = [PublishHTTPPushError.httpPushError().get_error_description()]


class HTTPPushService(PublishService):
    """HTTP Publish Service."""
    headers = {"Content-type": "application/json", "Accept": "application/json"}

    def _transmit(self, queue_item, subscriber):
        """
        @see: PublishService._transmit
        """
        item = json.loads(queue_item['formatted_item'])
        item['guid'] = item[config.ID_FIELD]
        del item[config.ID_FIELD]

        assets_url = queue_item.get('destination', {}).get('config', {}).get('assets_url')
        try:
            self._copy_published_media_files(json.loads(queue_item['formatted_item']), assets_url)
        except Exception as e:
            raise PublishHTTPPushError.httpPushError(e, queue_item.get('destination', {}))

        resource_url = queue_item.get('destination', {}).get('config', {}).get('resource_url')
        response = requests.post(resource_url, data=json.dumps(item), headers=self.headers)
        if response.status_code != requests.codes.created:  # @UndefinedVariable
            raise PublishHTTPPushError.httpPushError(Exception('Error pushing item %s' % response.text),
                                                     queue_item.get('destination', {}))

    def _copy_published_media_files(self, item, assets_url):
        """Copy the media files for the given item to the publish_items endpoint

        @param item: the item object
        @type item: dict
        @param assets_url: the url where the media can be uploaded
        @type assets_url: string
        """
        for name, rendition in item.get('renditions', {}).items():
            del item['renditions'][name]['href']
            if not self._media_exists(rendition['media'], assets_url):
                media = app.media.get(rendition['media'], resource='upload')
                files = {'media': (rendition['media'], media, rendition.get('mimetype') or rendition['mime_type'])}
                response = requests.post(assets_url, files=files, data={'media_id': rendition['media']})
                if response.status_code != requests.codes.created:  # @UndefinedVariable
                    raise Exception('Error pushing item %s media file %s' % (item._id, rendition['media']))

    def _media_exists(self, media_id, assets_url):
        """Returns true if the media with the given id exists at the service identified by assets_url.
        Returns false otherwise. Raises Exception if the error code was not 200 or 404

        @param media_id: the media identifier
        @type media_id: string
        @param assets_url: the url of the assest service
        @type assets_url: string
        @return: bool
        """
        response = requests.get('%s/%s' % (assets_url, media_id))
        if response.status_code not in (requests.codes.ok, requests.codes.not_found):  # @UndefinedVariable
            raise Exception('Error querying the assets service %s' % assets_url)
        return response.status_code == requests.codes.ok  # @UndefinedVariable

register_transmitter('http_push', HTTPPushService(), errors)
